#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<string.h>
#include	<sys/utsname.h>
#include	"lxrun.h"
#include	"linux/utsname.h"
#include	"linux/sysinfo.h"

extern char *linux_uname;

/*
 * administrative system calls
 */

/*
 *	olduname
 *	uname
 *	newuname
 *	swapon		(NOT IMPLEMENTED)
 *	swapoff		(NOT IMPLEMENTED)
 *	reboot		(NOT IMPLEMENTED)
 *	sysinfo
 *	adjtimex	(NOT IMPLEMENTED)
 *	quotactl	(NOT IMPLEMENTED)
 *	bdflush		(NOT IMPLEMENTED)
 */

int
lx_olduname(struct lx_oldold_utsname *lx_name)
{
	struct utsname name;
	char *uname_src;

	if (! verify_write(lx_name, sizeof(*lx_name)))
		return -1;

	if (uname(&name) < 0)
		return -1;

	uname_src = linux_uname ? linux_uname : name.sysname;

	strncpy(lx_name->sysname, uname_src, LX_OLD_UTS_LEN);
	lx_name->sysname[LX_OLD_UTS_LEN] = '\0';
	strncpy(lx_name->nodename, name.nodename, LX_OLD_UTS_LEN);
	lx_name->nodename[LX_OLD_UTS_LEN] = '\0';
	strncpy(lx_name->release, name.release, LX_OLD_UTS_LEN);
	lx_name->release[LX_OLD_UTS_LEN] = '\0';
	strncpy(lx_name->version, name.version, LX_OLD_UTS_LEN);
	lx_name->version[LX_OLD_UTS_LEN] = '\0';
	strncpy(lx_name->machine, name.machine, LX_OLD_UTS_LEN);
	lx_name->machine[LX_OLD_UTS_LEN] = '\0';

	return 0;
}

int
lx_uname(struct lx_old_utsname *lx_name)
{
	struct utsname name;
	char *uname_src;

	if (! verify_write(lx_name, sizeof(*lx_name)))
		return -1;

	if (uname(&name) < 0)
		return -1;

	uname_src = linux_uname ? linux_uname : name.sysname;

	strncpy(lx_name->sysname, uname_src, LX_NEW_UTS_LEN);
	lx_name->sysname[LX_NEW_UTS_LEN] = '\0';
	strncpy(lx_name->nodename, name.nodename, LX_NEW_UTS_LEN);
	lx_name->nodename[LX_NEW_UTS_LEN] = '\0';
	strncpy(lx_name->release, name.release, LX_NEW_UTS_LEN);
	lx_name->release[LX_NEW_UTS_LEN] = '\0';
	strncpy(lx_name->version, name.version, LX_NEW_UTS_LEN);
	lx_name->version[LX_NEW_UTS_LEN] = '\0';
	strncpy(lx_name->machine, name.machine, LX_NEW_UTS_LEN);
	lx_name->machine[LX_NEW_UTS_LEN] = '\0';

	return 0;
}

int
lx_newuname(struct lx_new_utsname *lx_name)
{
	int	r;

	if ((r = lx_uname((struct lx_old_utsname *)lx_name)) < 0)
		return r;

	return getdomainname(lx_name->domainname, LX_NEW_UTS_LEN);
}

int lx_swapon()		{ return enosys("swapon"); }
int lx_swapoff()	{ return enosys("swapoff"); }
int lx_reboot()		{ return enosys("reboot"); }

int
lx_sysinfo(struct lx_sysinfo *lx_info)
{
	if (! verify_write(lx_info, sizeof(*lx_info)))
		return -1;

	memset(lx_info, 0, sizeof(*lx_info));

	return 0;
}	

int lx_adjtimex()	{ return enosys("adjtimex"); }
int lx_quotactl()	{ return enosys("quotactl"); }
int lx_bdflush()	{ return enosys("bdflush"); }
