#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% %Q% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<sys/types.h>
#include	<string.h>
#if defined(UNIXWARE) || defined(UW7)
#include	<sys/statvfs.h>
#else
#include	<sys/statfs.h>
#endif

#include	"linux/statfs.h"
#include	"lxrun.h"

/*
 * filesystem specific system calls
 */

/*
 *	mount		(NOT IMPLEMENTED)
 *	umount		(NOT IMPLEMENTED)
 *	statfs
 *	fstatfs
 *	setfsuid	(NOT IMPLEMENTED)
 *	setfsgid	(NOT IMPLEMENTED)
 */

int lx_mount()		{ return enosys("mount"); }
int lx_umount()		{ return enosys("umount"); }



#if defined(UNIXWARE) || defined(UW7) /* better trigger on if struct statvfs exists */

static int
copy_out_new_statfs(struct statvfs *vfs, struct lx_statfs *to)
{
	if (! verify_write(to, sizeof(*to)))
		return -1;

	(void) memset((void *)to, 0, sizeof(*to));

	to->f_type	=	EXT_SUPER_MAGIC;
	to->f_bsize	=	vfs->f_bsize;
	to->f_blocks	=	vfs->f_blocks;
	to->f_bfree	=	vfs->f_bfree;
	to->f_bavail	=	vfs->f_bavail;
	to->f_files	=	vfs->f_files; 
	to->f_ffree	=	vfs->f_ffree;
	to->f_fsid.val[0] =	vfs->f_fsid;
	to->f_fsid.val[1] =	0;
	to->f_namelen	=	vfs->f_namemax;

	return 0;
}

int
lx_statfs(const char *path, struct lx_statfs *buf)
{
	struct statvfs	vfs;

	if (statvfs(path, &vfs) == -1)
		return -1;
	return copy_out_new_statfs(&vfs, buf);
}
int
lx_fstatfs(int fd, struct lx_statfs *buf)
{
	struct statvfs	vfs;

	if (fstatvfs(fd, &vfs) == -1)
		return -1;

	return copy_out_new_statfs(&vfs, buf);
}

#else   /* ! UW7 */

static int
copy_out_new_statfs(struct statfs *from, struct lx_statfs *to)
{
	if (! verify_write(to, sizeof(*to)))
		return -1;

	(void) memset((void *)to, 0, sizeof(*to));

	to->f_type =	from->f_fstyp;
	to->f_bsize =	from->f_bsize;
	to->f_blocks =	from->f_blocks;
	to->f_bfree =	from->f_bfree;	/* blocks avail. in filesystem */
	to->f_bavail =	from->f_bfree;	/* blocks avail. to non-superuser */
	to->f_files =	from->f_files;
	to->f_ffree =	from->f_ffree;

	/*
	 * Fields that are undef for a particular file system are set to -1.
	 * (see statfs(2)).
	 */
	to->f_namelen = -1;

	return 0;
}


lx_statfs(const char *path, struct lx_statfs *buf)
{
	int ret;
	struct statfs nbuf;

	if ((ret = statfs(path, &nbuf, sizeof(struct statfs), 0)) == 0)
		ret = copy_out_new_statfs(&nbuf, buf);

	return ret;
}

int
lx_fstatfs(int fd, struct lx_statfs *buf)
{
	int ret;
	struct statfs nbuf;

	if ((ret = fstatfs(fd, &nbuf, sizeof(struct statfs), 0)) == 0)
		ret = copy_out_new_statfs(&nbuf, buf);

	return ret;
}
#endif  /* ! UW7 */	

int lx_setfsuid()	{ return enosys("setfsuid"); }
int lx_setfsgid()	{ return enosys("setfsgid"); }
