#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<stdlib.h>
#ifdef DSP_GETOPTR_HACK
# include	<sys/soundcard.h>
#endif
#include	<stdio.h>
#include	<sys/termio.h>
#include	<termios.h>
#include	<unistd.h>
#include	<string.h>
#include	<errno.h>
#include	<sys/time.h>
#include	<sys/select.h>
#include        <fcntl.h>

#ifdef UW7
#   define __IOCTL_VERSIONED__
#   define _EFTSAFE
#endif
#include	<sys/socket.h>

#if defined(UNIXWARE) || defined(UW7) || defined(SOLARIS)
# include	<sys/filio.h>
# include	<sys/sockio.h>
# include	<net/if.h>
#else
# include	<sys/net/if.h>
#endif

#ifdef UW7  /* fm osocket.h BSD compat */
#define OIOCPARM_MASK	0x7f	/* parameters must be < 128 bytes */
#define OIOC_VOID	0x20000000	/* no parameters */
#define OIOC_OUT	0x40000000	/* copy out parameters */
#define OIOC_IN		0x80000000	/* copy in parameters */
#define OIOC_INOUT	(OIOC_IN|OIOC_OUT)
#define _OIOS(x,y)	(OIOC_VOID|(x<<8)|y)
#define _OIOSR(x,y,t)	(OIOC_OUT|((sizeof(t)&OIOCPARM_MASK)<<16)|(x<<8)|y)
#define _OIOSW(x,y,t)	(OIOC_IN|((sizeof(t)&OIOCPARM_MASK)<<16)|(x<<8)|y)
#define	_OIOSWR(x,y,t)	(OIOC_INOUT|((sizeof(t)&OIOCPARM_MASK)<<16)|(x<<8)|y)

#define OFIONREAD	_OIOSR('S', 8, int)	/* BSD compatibilty */
#define OFIONBIO	_OIOSW('S', 9, int)	/* BSD compatibilty */
#endif  /* UW7 */

#if defined(OSR5)
# include       <sys/vtkd.h>
# include       <sys/console.h>
# include       <sys/vid.h>
#endif

#include	"lxrun.h"
#include	"linux/socket.h"
#include	"linux/termios.h"
#include	"linux/vtkd.h"
#include	"devmap.h"
#include	"trace.h"

#ifndef TCGETS
# define TCGETS		XCGETA
# define TCSETS		XCSETA
# define TCSETSW	XCSETAW
# define TCSETSF	XCSETAF
#endif

extern int tioclinux(int fd, void *arg);

/*
 * ioctl() related system calls
 */

/*
 *	ioctl 		(PARTLY IMPLEMENTED)
 */


int lx_ioctl_default();
int lx_ioctl_audio();

static const char *lx_iocstr(int lx_cmd);

/*
** Initialize the device-to-ioctl-handler map
*/
void
devmap_init(void)
{
	int i;

	for(i=0; i<NMAJ; i++)
		devmap_add_ioctl_handler(lx_ioctl_default, i);


	/* *** PUT REGISTRATION FOR NEW IOCTL HANDLERS HERE *** */

	devmap_add_ioctl_handler(lx_ioctl_audio, devmap_major("/dev/dsp"));
}

int
lx_ioctl_default(int fd, int lx_cmd, unsigned long lx_arg, int maj, int min)
{
	int		cmd;
	int		argtype;

	IOCTL_TRACE_MSG(("TRACE: ioctl(%d, %s, %#x, maj=%d/min=%d)\n",
				fd, lx_iocstr(lx_cmd), lx_arg, maj, min));

	switch (lx_cmd)
	{
		case LX_TCGETS: 
		{
			int rv ;
			struct termios tio ;
			struct lx_termios* lx_tio=(struct lx_termios *)lx_arg;

			if(!verify_write(lx_tio, sizeof(*lx_tio)))
			{
				/* verify_write() set errno to EFAULT */
				return -1;
			}

			rv = ioctl(fd, TCGETS, &tio);
			if (rv)
			    return rv;

			memset(lx_tio, 0, sizeof(struct lx_termios));

			/*
			 * This isn't really safe, since each side has some
			 * flags the other doesn't.  But the cost of doing a
			 * bitwise test for each one it just too horrible
			 * for words.
			 */
			lx_tio->c_iflag = tio.c_iflag;
			lx_tio->c_oflag = tio.c_oflag;
			lx_tio->c_cflag = tio.c_cflag;
			lx_tio->c_lflag = tio.c_lflag;
        
			if (tio.c_lflag & ICANON)
			{
				lx_tio->c_cc[LX_VEOF]	= tio.c_cc[VEOF] ;
				lx_tio->c_cc[LX_VEOL]	= tio.c_cc[VEOL] ;
			}
			else 
			{
				lx_tio->c_cc[LX_VMIN]	= tio.c_cc[VMIN] ;
				lx_tio->c_cc[LX_VTIME]	= tio.c_cc[VTIME] ;
			}
        
			lx_tio->c_cc[LX_VINTR]	= tio.c_cc[VINTR] ;
			lx_tio->c_cc[LX_VQUIT]	= tio.c_cc[VQUIT] ;
			lx_tio->c_cc[LX_VERASE] = tio.c_cc[VERASE] ;
			lx_tio->c_cc[LX_VKILL]	= tio.c_cc[VKILL] ;
			lx_tio->c_cc[LX_VSWTC]	= tio.c_cc[VSWTCH] ;
			lx_tio->c_cc[LX_VSTART] = tio.c_cc[VSTART] ;
			lx_tio->c_cc[LX_VSTOP]	= tio.c_cc[VSTOP] ;
			lx_tio->c_cc[LX_VSUSP]	= tio.c_cc[VSUSP] ;
			lx_tio->c_cc[LX_VEOL]	= tio.c_cc[VEOL] ;
			return 0 ;
		}

		case LX_TCSETS:
		case LX_TCSETSW:
		case LX_TCSETSF:
		{
			struct termios tio ;
			struct lx_termios* lx_tio=(struct lx_termios *)lx_arg;
			long sco_cmd = -1;

			if (!verify_read(lx_tio, sizeof(*lx_tio)))
			{
				/* verify_read() set errno to EFAULT */
				return -1;
			}

			switch (lx_cmd) 
			{
				case LX_TCSETS:  sco_cmd = TCSETS;	break;
				case LX_TCSETSW: sco_cmd = TCSETSW;	break;
				case LX_TCSETSF: sco_cmd = TCSETSF;	break;
			}

			memset(&tio, 0, sizeof(struct termios)); 

			/*
			 * This isn't really safe, since each side has some
			 * flags the other doesn't.  But the cost of doing a
			 * bitwise test for each one it just too horrible
			 * for words.
			 */
        
			tio.c_iflag = lx_tio->c_iflag;
			tio.c_oflag = lx_tio->c_oflag;
			tio.c_cflag = lx_tio->c_cflag;
			tio.c_lflag = lx_tio->c_lflag;
        
			/* On the SCO side, these overlap (sigh) */

			if (lx_tio->c_lflag & LX_ICANON) 
			{
				tio.c_cc[VEOF]	= lx_tio->c_cc[LX_VEOF] ;
				tio.c_cc[VEOL]	= lx_tio->c_cc[LX_VEOL] ;
			}
			else
			{
				tio.c_cc[VTIME] = lx_tio->c_cc[LX_VTIME] ;
				tio.c_cc[VMIN]	= lx_tio->c_cc[LX_VMIN] ;
			}

			tio.c_cc[VINTR]  = lx_tio->c_cc[LX_VINTR] ;
			tio.c_cc[VQUIT]  = lx_tio->c_cc[LX_VQUIT] ;
			tio.c_cc[VERASE] = lx_tio->c_cc[LX_VERASE] ;
			tio.c_cc[VKILL]  = lx_tio->c_cc[LX_VKILL] ;
			tio.c_cc[VSWTCH] = lx_tio->c_cc[LX_VSWTC] ;
			tio.c_cc[VSTART] = lx_tio->c_cc[LX_VSTART] ;
			tio.c_cc[VSTOP]  = lx_tio->c_cc[LX_VSTOP] ;
			tio.c_cc[VSUSP]  = lx_tio->c_cc[LX_VSUSP] ;

			return ioctl(fd, sco_cmd, &tio);
		}

		case LX_TCGETA:
		{
		       int rv ;
		       struct termio tio ;
		       struct lx_termio* lx_tio = (struct lx_termio *) lx_arg ;
		       rv = ioctl(fd, TCGETA, &tio);
		       if (rv)
			return rv;

		       if (! verify_write(lx_tio, sizeof(*lx_tio)))
		       {
			    /* verify_write() set errno to EFAULT */
			    return -1;
			}

		       memset(lx_tio, 0, sizeof(struct lx_termio));

			/*
			* This isn't really safe, since each side has some
			* flags the other doesn't.  But the cost of doing a
			* bitwise test for each one it just too horrible
			* for words.
			*/

			lx_tio->c_iflag = tio.c_iflag;
			lx_tio->c_oflag = tio.c_oflag;
			lx_tio->c_cflag = tio.c_cflag;
			lx_tio->c_lflag = tio.c_lflag;

			if (tio.c_lflag & ICANON)
			{
			       lx_tio->c_cc[LX_VEOF]   = tio.c_cc[VEOF] ;
			       lx_tio->c_cc[LX_VEOL]   = tio.c_cc[VEOL] ;
			}
			else
			{
			       lx_tio->c_cc[LX_VMIN]   = tio.c_cc[VMIN] ;
			       lx_tio->c_cc[LX_VTIME]  = tio.c_cc[VTIME] ;
			}
			lx_tio->c_cc[LX_VINTR]	= tio.c_cc[VINTR] ;
			lx_tio->c_cc[LX_VQUIT]	= tio.c_cc[VQUIT] ;
			lx_tio->c_cc[LX_VERASE] = tio.c_cc[VERASE] ;
			lx_tio->c_cc[LX_VKILL]	= tio.c_cc[VKILL] ;
			lx_tio->c_cc[LX_VSWTC]	= tio.c_cc[VSWTCH] ;
			lx_tio->c_cc[LX_VSTART] = tio.c_cc[VSTART] ;
			lx_tio->c_cc[LX_VSTOP]	= tio.c_cc[VSTOP] ;
			lx_tio->c_cc[LX_VSUSP]	= tio.c_cc[VSUSP] ;
			lx_tio->c_cc[LX_VEOL]	= tio.c_cc[VEOL] ;
			return 0 ;
		}

		case LX_TCSETA:
		case LX_TCSETAW:
		case LX_TCSETAF:
		{
		       struct termio tio ;
		       struct lx_termio* lx_tio = (struct lx_termio *) lx_arg ;
		       long sco_cmd = -1 ;

		       if (! verify_read(lx_tio, sizeof(*lx_tio)))
			       return -1;

		       switch (lx_cmd)
			       {
			       case LX_TCSETA:  sco_cmd = TCSETA ;  break;
			       case LX_TCSETAW: sco_cmd = TCSETAW ; break;
			       case LX_TCSETAF: sco_cmd = TCSETAF ; break;
			       }

		       memset(&tio, 0, sizeof(struct termio));

	                /*
	                 * This isn't really safe, since each side has some
	                 * flags the other doesn't.  But the cost of doing a
	                 * bitwise test for each one it just too horrible
	                 * for words.
	                 */
	                 tio.c_iflag = lx_tio->c_iflag;
	                 tio.c_oflag = lx_tio->c_oflag;
	                 tio.c_cflag = lx_tio->c_cflag;
	                 tio.c_lflag = lx_tio->c_lflag;

		       /* On the SCO side, these overlap (sigh) */

		       if (lx_tio->c_lflag & LX_ICANON)
			       {
			       tio.c_cc[VEOF]  = lx_tio->c_cc[LX_VEOF] ;
			       tio.c_cc[VEOL]  = lx_tio->c_cc[LX_VEOL] ;
			       }
		       else
			       {
			       tio.c_cc[VTIME] = lx_tio->c_cc[LX_VTIME] ;
			       tio.c_cc[VMIN]  = lx_tio->c_cc[LX_VMIN] ;
			       }

		       tio.c_cc[VINTR]	= lx_tio->c_cc[LX_VINTR] ;
		       tio.c_cc[VQUIT]	= lx_tio->c_cc[LX_VQUIT] ;
		       tio.c_cc[VERASE]	= lx_tio->c_cc[LX_VERASE] ;
		       tio.c_cc[VKILL]	= lx_tio->c_cc[LX_VKILL] ;
		       tio.c_cc[VSWTCH]	= lx_tio->c_cc[LX_VSWTC] ;
		       tio.c_cc[VSTART]	= lx_tio->c_cc[LX_VSTART] ;
		       tio.c_cc[VSTOP]	= lx_tio->c_cc[LX_VSTOP] ;
		       tio.c_cc[VSUSP]	= lx_tio->c_cc[LX_VSUSP] ;

		       return ioctl(fd, sco_cmd, &tio);
		}

		case LX_TCSBRK:
		case LX_TCXONC:
		case LX_TCFLSH:
		case LX_TIOCEXCL:
	        case LX_TIOCNXCL:
		    {
			char msg[64];
			snprintf(msg, sizeof(msg),
				 "Unimplemented ioctl(%s) 0x%x",
				 lx_iocstr(lx_cmd), lx_cmd );
			return enoany(ENOSYS, msg);
		    }

		case LX_TIOCGPGRP:
			{
			pid_t pgrp=tcgetpgrp(fd);
			if(pgrp == -1)
				return -1;

			*((pid_t *)lx_arg) = pgrp;
			return 0;
			}

		case LX_TIOCSPGRP:
			return tcsetpgrp(fd, *((pid_t *)lx_arg));

		case LX_TIOCGWINSZ:
			return ioctl(fd, TIOCGWINSZ, lx_arg);

		case LX_TIOCSWINSZ:
			break;

		case LX_FIONREAD:
#ifdef OFIONREAD
	        {
	            int		r;

	            if (((r = ioctl(fd, OFIONREAD, lx_arg)) != -1) ||
	                    (errno != EINVAL))
	                return r;
	        }
	            IOCTL_TRACE_MSG(("TRACE: ioctl Retrying FIONREAD\n"));
#endif
		return ioctl(fd, FIONREAD, lx_arg);
			
		case LX_FIONBIO:
#ifdef OFIONBIO
	        {
	                int		r;

	                if (((r = ioctl(fd, OFIONBIO, lx_arg)) != -1) ||
	                   (errno != EINVAL))
	                return r;
	        }   
		        IOCTL_TRACE_MSG(("TRACE: ioctl Retrying FIONBIO\n"));
#endif
			return ioctl(fd, FIONBIO, lx_arg);

	/*
	 * It is a gift that the structures seem to match
	 * "well enough"
	 */

		case LX_SIOCGIFCONF:
			return ioctl(fd, SIOCGIFCONF, lx_arg);

		case LX_SIOCGIFFLAGS:
			return ioctl(fd, SIOCGIFFLAGS, lx_arg);
 
		case LX_SIOCGIFADDR:
			return ioctl(fd, SIOCGIFADDR, lx_arg);
 
		case LX_SIOCGIFNETMASK:
			return ioctl(fd, SIOCGIFNETMASK, lx_arg);
 
		case LX_SIOCGIFBRDADDR:
			return ioctl(fd, SIOCGIFBRDADDR, lx_arg);

		case LX_TIOCLINUX:
			return tioclinux(fd, (void *)lx_arg);

#ifdef OSR5
		case LX_KDGETMODE:
			return ioctl(fd, KDGETMODE, lx_arg);

		case LX_KDSETMODE:
			return ioctl(fd, KDSETMODE, lx_arg);

		case LX_VT_GETSTATE:	/* partially implemented */
			{
			struct lx_vt_stat *stat = (struct lx_vt_stat *)lx_arg;
			struct vid_info vi;
			int tmpfd;

			/* 16-bit bitmask telling us which multiscreens are
			** usable.  We'll assume the default OSR5 configuration
			** -- i.e., we have 12 active multiscreens.  */
			stat->v_state = 0x0fff;

			/* # of active multiscreen (1-based) */
			/*
			** We have a slight problem here in that we must
			** be root to open /dev/vga -- necessary to determine
			** the current multiscreen -- and on Linux, VT_GETSTATE
			** is not a privileged ioctl.  Oh well.
			**
			** Also, we're assuming that everyone these days has
			** vga.  Technically, we should be doing a CONS_CURRENT
			** to find out what kind of display we have before
			** blindly opening /dev/vga.
			*/
			if((tmpfd = open("/dev/vga", O_RDONLY)) == -1)
				return -1;
			vi.size = sizeof(struct vid_info);
			ioctl(tmpfd, CONS_GETINFO, &vi);
			close(tmpfd);
			stat->v_active = vi.m_num+1;
				
			/* ??? */
			/* stat->v_signal = */

			}
#else
		case LX_KDGETMODE:	
		case LX_KDSETMODE:
		case LX_VT_GETSTATE:
		    {
			char msg[64];
			snprintf(msg, sizeof(msg),
				 "Unimplemented ioctl(%s) 0x%x",
				 lx_iocstr(lx_cmd));
			return enoany(ENOSYS, msg);
		    }
#endif	/* !OSR5 */

		case LX_TIOCSCTTY:
#if defined(SOLARIS)
			/*
			 * SysV semantics for controlling terminals say that
			 * the first terminal device not already associated
			 * with a session, and not opened with O_NOCTTY, will
			 * become the controlling tty for the session.
			 *
			 * Presumably, if a process were calling TIOCSCTTY
			 * on a file descriptor, it will have opened it
			 * without O_NOCTTY, so it will have already become
			 * the controlling tty, and 0 would be the correct
			 * return value.
			 * 
			 * XXX: This is incorrect if a process creates a new
			 * session, opens two terminal devices, and wants
			 * the second to be its controlling terminal.
			 * That's pretty hard to deal with.
			 */
			return 0;
#else
			/*FALLTHROUGH*/
#endif
	        case LX_TIOCSETD:
		case LX_TIOCGETD:
		case LX_FIONCLEX:
		case LX_FIOCLEX:
		case LX_FIOASYNC:
		case LX_TIOCOUTQ:
		case LX_TIOCSTI:
		case LX_TIOCMGET:
		case LX_TIOCMBIS:
		case LX_TIOCMBIC:
		case LX_TIOCMSET:
		case LX_TIOCGSOFTCAR:
		case LX_TIOCSSOFTCAR:
		case LX_TIOCCONS:
		case LX_TIOCGSERIAL:
		case LX_TIOCSSERIAL:
		case LX_TIOCPKT:
		case LX_TIOCNOTTY:
		case LX_TCSBRKP:
		case LX_TIOCTTYGSTRUCT:
		case LX_TIOCSERCONFIG:
		case LX_TIOCSERGWILD:
		case LX_TIOCSERSWILD:
		case LX_TIOCGLCKTRMIOS:
		case LX_TIOCSLCKTRMIOS:
		case LX_TIOCSERGSTRUCT:
		case LX_TIOCSERGETLSR:
		case LX_TIOCSERGETMULTI:
		case LX_TIOCSERSETMULTI:
	        default:
		    {
			char msg[64];
			/* IOCTL_TRACE_MSG(("TRACE:...K") */
			snprintf(msg, sizeof(msg),
				 "Unimplemented ioctl(%s) 0x%x",
				 lx_iocstr(lx_cmd), lx_cmd);
			return enoany(ENOSYS, msg);
		    }
	}
}

/*
 * ioctl mapping for OSS audio drivers
 */

int
lx_ioctl_audio(int fd, int lx_cmd, unsigned long lx_arg, int maj, int min)
{
	unsigned char	cmdkey = (lx_cmd >> 8) & 0xff;
#ifdef DSP_GETOPTR_HACK
	static int dsp_bytes, dsp_byte_incr = -1;
#endif

	if(cmdkey == 'P' ||	/* DSP */
	   cmdkey == 'M' ||	/* Mixer */
	   cmdkey == 'm' ||	/* MIDI */
	   cmdkey == 'C' ||	/* Co-Processor */
	   cmdkey == 'T' ||	/* Timer */
	   cmdkey == 'Q')	/* Sequencer */
        {
		/*
	         * Since pretty much everyone uses the 4Front Technologies
	         * drivers, there's not much conversion to do.  (There
	         * wouldn't be any, but the direction information is
	         * handled differently on Linux.)
		 */
		unsigned int sco_cmd, dirbits;
		int ret;

		/*
	         * TTD: This should be changed so we use macros from
	         * ioctl.h and/or soundcard.h instead of using hard-
	         * coded values.
		 */

		/* Collect direction bits from original command */
		dirbits = lx_cmd & 0xc0000000;

		/* Perform transformation */
		if(dirbits & 0x80000000)
		{
			dirbits &= ~0x80000000;
			dirbits |=  0x20000000;
		}

		/* Mask them back in */
		sco_cmd = (lx_cmd & 0x1ffffff) | dirbits;

		/* Call the audio ioctl */
		ret = ioctl(fd, sco_cmd, lx_arg);
#ifdef DSP_GETOPTR_HACK
		/* fudge some DSP_GETOPTR count info */
		/* This works around a bug in old 4Front/OSS drivers
		** in which DSP_GETOPTR returns garbage.  Programs
		** that require audio/video sync (e.g., rvplayer,
		** some games) may not work properly without this
		** if you have one of the buggy drivers.  This
		** particularly affects UW2 since the last OSS drivers
		** released for UW2 had this bug. */
		if (ret != 0)
			{
			/* make up for failed ioctls */
			if (sco_cmd == SNDCTL_DSP_SETFRAGMENT)
				{
				IOCTL_TRACE_MSG(("TRACE: ioctl(%d, "
					"SNDCTL_DSP_SETFRAGMENT, 0x%x) failed "
					"- ignoring\n",
					fd, *(unsigned int *)lx_arg));
				ret=0;
				}
			}
		/* hack non-failed ioctls */
		else if (sco_cmd == SNDCTL_DSP_RESET)
		{
			dsp_bytes=0;
			IOCTL_TRACE_MSG(("TRACE: ioctl(%d, SNDCTL_DSP_RESET)\n",
				fd));
		}
		else if (sco_cmd == SNDCTL_DSP_GETOPTR)
		{
		      char *incstr;
		      count_info *infp = (count_info *)lx_arg;

		      /* Use byte increment specified in environment,
			 or default of 1000 */
		      if (dsp_byte_incr == -1 &&
			  (incstr = getenv("LXRUN_DSP_BYTE_INCR")) != NULL)
				dsp_byte_incr = atoi(incstr);
		      if (dsp_byte_incr <= 0)
				dsp_byte_incr = 1000;

		      /* Increment the byte counter on each call
			 (rvplayer only uses the byte count - may need
			  to fudge blocks/ptr for other programs) */
		      infp->bytes = (dsp_bytes += dsp_byte_incr);

		      IOCTL_TRACE_MSG(("TRACE: ioctl(%d, SNDCTL_DSP_GETOPTR, bytes=%d/blocks=%d/ptr=%d)\n",
			      fd, infp->bytes, infp->blocks, infp->ptr));
		}
#endif /* DSP_GETOPTR_HACK */
		return ret;
	}

        enoany(ENOSYS, "Unimplemented audio ioctl cmdkey" );
}


typedef struct
{
    int		value;
    const char	*name;
} i_name_t;

#define I_NM(x)		{ LX_##x, #x }
#define I_NM_END	{ NULL }

static const char *
lx_iocstr(int lx_cmd)
{
    static const i_name_t	iocs[] =
		{
	I_NM(TCGETS),
	I_NM(TCSETS),
	I_NM(TCSETSW),
	I_NM(TCSETSF),
	I_NM(TCGETA),
	I_NM(TCSETA),
	I_NM(TCSETAW),
	I_NM(TCSETAF),
	I_NM(TCSBRK),
	I_NM(TCXONC),
	I_NM(TCFLSH),
	I_NM(TIOCEXCL),
	I_NM(TIOCNXCL),
	I_NM(TIOCGPGRP),
	I_NM(TIOCSPGRP),
	I_NM(TIOCGWINSZ),
	I_NM(TIOCSWINSZ),
	I_NM(FIONREAD),
	I_NM(FIONBIO),
	I_NM(SIOCGIFCONF),
	I_NM(SIOCGIFFLAGS),
	I_NM(SIOCGIFADDR),
	I_NM(SIOCGIFNETMASK),
	I_NM(SIOCGIFBRDADDR),
	I_NM(TIOCSETD),
	I_NM(TIOCGETD),
	I_NM(FIONCLEX),
	I_NM(FIOCLEX),
	I_NM(FIOASYNC),
	I_NM(TIOCOUTQ),
	I_NM(TIOCSTI),
	I_NM(TIOCSCTTY),
	I_NM(TIOCMGET),
	I_NM(TIOCMBIS),
	I_NM(TIOCMBIC),
	I_NM(TIOCMSET),
	I_NM(TIOCGSOFTCAR),
	I_NM(TIOCSSOFTCAR),
	I_NM(TIOCLINUX),
	I_NM(TIOCCONS),
	I_NM(TIOCGSERIAL),
	I_NM(TIOCSSERIAL),
	I_NM(TIOCPKT),
	I_NM(TIOCNOTTY),
	I_NM(TCSBRKP),
	I_NM(TIOCTTYGSTRUCT),
	I_NM(TIOCSERCONFIG),
	I_NM(TIOCSERGWILD),
	I_NM(TIOCSERSWILD),
	I_NM(TIOCGLCKTRMIOS),
	I_NM(TIOCSLCKTRMIOS),
	I_NM(TIOCSERGSTRUCT),
	I_NM(TIOCSERGETLSR),
	I_NM(TIOCSERGETMULTI),
	I_NM(TIOCSERSETMULTI),
	I_NM_END
    };
    static char		buf[32];
    const i_name_t	*ip;

    for (ip = iocs; ip->name; ++ip)
	if (ip->value == lx_cmd)
	    return ip->name;

    sprintf(buf, "<Unknown %#x>", lx_cmd);
    return buf;
}

