#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<sys/types.h>
#include	<unistd.h>

#ifdef UW7
#   define _EFTSAFE
#endif
#include	<sys/socket.h>

#include	"lxrun.h"
#include	"linux/msg.h"
#include	"linux/sem.h"
#include	"linux/shm.h"

int lx_pipe(int filedes[2]);
extern int lx_semop(int semid, struct lx_sembuf *sops, unsigned nsops);
extern int lx_semget(key_t key, int nsems, int semflg);
extern int lx_semctl(int semid, int semnum, int cmd, void *arg);
extern int lx_msgsnd(int msqid, struct lx_msgbuf *msgp, int msgsz, int msgflg);
extern int lx_msgrcv(int msqid, struct lx_msgbuf *msgp, int msgsz, long msgtyp,
								int msgflg);
extern int lx_msgget(key_t key, int msgflg);
extern int lx_msgctl(int msqid, int cmd, struct lx_msqid_ds *buf);
extern int lx_shmat(int shmid, char *shmaddr, int lx_shmflg, void **result);
extern int lx_shmdt(char *shmaddr);
extern int lx_shmget(key_t key, int size, int lx_flag);
extern int lx_shmctl(int shmid, int cmd, struct lx_shmid_ds *buf);


#if defined(OSR5) || defined(UW7)
#define UNIDIRECTIONAL_PIPES
#endif

/*
** Since Linux pipes can be bi-directional, use a socketpair to
** implement a pipe.
*/
int
lx_pipe(int filedes[2])
{
#ifndef UNIDIRECTIONAL_PIPES
	return socketpair(AF_UNIX, SOCK_STREAM, 0, filedes);
#else
	return pipe(filedes);
#endif
}


int
lx_ipc(unsigned call, int arg1, int arg2, int arg3, void *arg4)
{
	int	version	= call >> 16;

	/*
	 * Linux binaries should all be version 0
	 * version 1 is only used by the iBCS2 emulatior
	 */
	if (version != 0)
	{
		return enosys("ipc(version != 0)");
	}

	switch (call & 0xffff)
	{
		case LX_SEMOP:
			return lx_semop(arg1, arg4, arg2);

		case LX_SEMGET:
			return lx_semget(arg1, arg2, arg3);

		case LX_SEMCTL: 
			if (!arg4)
			    return enoany(EINVAL,
					  "lx_ipc - LX_SEMCTL no arg4");

	    return lx_semctl(arg1, arg2, arg3, (void *)*(long *)arg4);

		case LX_MSGSND:
			return lx_msgsnd(arg1, arg4, arg2, arg3);

		case LX_MSGRCV:
		{
			struct lx_ipc_kludge tmp;

			if (!arg4)
			{
			    return enoany(EINVAL,"lx_ipc - LX_MSGRCV no arg4");
			}

			tmp	= *(struct lx_ipc_kludge *)arg4;
			return lx_msgrcv(arg1, tmp.ptr, arg2, tmp.type, arg3);
		}

		case LX_MSGGET:
			return lx_msgget(arg1, arg2);

		case LX_MSGCTL:
			return lx_msgctl(arg1, arg2, arg4);

		case LX_SHMAT:
	    return lx_shmat(arg1, arg4, arg2, (void **)arg3);

	case LX_SHMDT: 
	    return lx_shmdt(arg4);

	case LX_SHMGET:
	    return lx_shmget(arg1, arg2, arg3);

	case LX_SHMCTL:
	    return lx_shmctl(arg1, arg2, arg4);
    }

    return enoany(EINVAL, "lx_ipc - unknown callflag");
}
