#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% %Q% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include <sys/types.h>

#if !defined(LXRUN_H)
#define LXRUN_H

#ifdef UW7
#define	UNIXWARE	1
#endif

#define	BLOCK_SIZE	1024

#if !defined(PAGE_SIZE)
#define	PAGE_SIZE	4096
#endif

#ifndef PATH_MAX
#define	PATH_MAX	1024
#endif

#ifndef SOLARIS
#define	uintptr_t	ulong_t
#endif

#define BLOCK_ALIGN(x)	(((ulong_t)(x) + BLOCK_SIZE - 1) & ~(BLOCK_SIZE - 1))
#define PAGE_ALIGN(x)	(((ulong_t)(x) + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1))

extern int	run_program(const char *progname, char **argv, char **envp);
extern void	lxstart(void *start, void *argv, void *envp, void *auxv);
extern void	*load_library(const char *libname);
extern int	is_linux_exec(const char *filename);
extern int	init_signals(void);

extern int	lx_errno(int err);
extern int	lx_pathname(const char *lx_path, char *os_path, int len);
extern int	is_remappable(int sysnum, int argnum);
#if defined(PATHMAPFILE) && defined(PHANTOM_MAP_FILE)
extern void	remove_phantom_file(void);
#endif

extern void	*dz_mmap(void *, size_t, int, int, off_t);
extern void	dz_invalidate(int);

extern int	getbyte(const void *addr);
extern int	putbyte(void *addr, int value);

extern int	verify_read(const void *addr, unsigned len);
extern int	verify_write(void *addr, unsigned len);
extern int	verify_readstr(const char *addr);

extern void	fatal(int err, char *format, ...);
extern void	warning(int err, char *format, ...);
extern int	enosys(const char *msg);
extern int	enoany(int r_errno, const char *msg);

#define EACCESS(p, mode)	(access(p, (mode) | 010) == 0)
#ifdef USE_EACCESS
#undef EACCESS
#include <libgen.h>
#define EACCESS(p, mode)	(eaccess(p, mode) == 0)
#endif

void	tprintf(char *format, ...);
void	trace_syscall_entry(int n, int *regs);
void	trace_syscall_result(int n, int rval, int err);
void	trace_signal(int sig, void *info, void *context);
void	trace_fork(void);

struct lx_sigcontext_struct;
extern void	trace_reg_changes(unsigned long *regs,
		    struct lx_sigcontext_struct *lx_context);
extern void	trace_unexpected_sigsegv(void **regs, uchar_t *eip);

extern int	getdomainname(char *name, int namelen);
extern void	*alloca(unsigned);
extern int	is_interp_script(const char *filename, char *interp,
		    int size, char *arg, int argsize);
extern void	chkpath_lxprog(const char *pathenv, char **lxprog);

extern char	*linux_cmdline;

#endif
