# lxrun Makefile 1998, 1999 SCO Inc.
# handles builds for Osr5 (OS build tree) UW7 (OS build tree)
# and stand alone (incl Solaris)
#
#

# for standalone build
#  run from /usr/local/bin and install system into /usr/local/linux
LOCAL_PATH              =       /local
LXROOT_PREFIX           =       /usr$(LOCAL_PATH)
# otherwise System build+install
#   run from /usr/bin and install into /usr/lib/linux
#LXROOT_PREFIX              =       /usr/lib

#
# for OpenServer in OS build tree
#   run from /usr/bin and install into /usr/lib/linux
#
# include ../make.inc

#
# for UnixWare 7 in OS build tree
#   run from /usr/bin and install into /usr/lib/linux
#
#include $(CMDRULES)
#SYSPREFIX               =       $(ROOT)/$(MACH)

#
# Common defines
#
MAKEFILE		=	lxrun.mk
PROG			=	lxrun
LXRUN_LOADER		=	ld-lxrun.so.1
GENERIC_LINUX_LOADER	= 	ld-linux.so.1
NEW_GENERIC_LINUX_LOADER= 	ld-linux.so.2

LINUX_IMPORT_DIR	=	from_linux
NAME_SERVICE		=	nsswitch.conf
NEW_LINUX_LOADER	= 	ld-2.0.7.so
NEW_LINUX_LOADER_ENC	= 	ld-2.0.7.so.enc
LINUX_LOADER		= 	ld-linux.so.1.7.14
LINUX_LOADER_ENC	= 	ld-linux.so.1.7.14.enc
CACHE			=	ld.so.cache
CACHE_ENC		=	ld.so.cache.enc
LINUX_LOCALE_TAR_Z	=	linuxlocale.tar.Z
LINUX_LOCALE_ENC	=	linuxlocale.tar.Z.enc
LINUX_LIBS_TAR_Z	=	linuxlibs.tar.Z
LINUX_LIBS_TAR_ENC	=	linuxlibs.tar.Z.enc

DECODE_DIR		=	decode_dir
ENCODE			=	$(DECODE_DIR)/encode -c 33-36,38-126
DECODE			=	$(DECODE_DIR)/decode

PATHMAP			=	PATHMAP
LXROOT			=	$(SYSPREFIX)$(LXROOT_PREFIX)/linux
REAL_LXROOT		=	$(LXROOT_PREFIX)/linux
TRACE_DIR		=	/tmp

BINDIR			=	$(LXROOT)/bin
LIBDIR			=	$(LXROOT)/lib
ETCDIR			=	$(LXROOT)/etc
EMULATOR		=	$(REAL_LXROOT)/bin/$(PROG)

SYSLIBDIR		=	$(SYSPREFIX)/lib
SYSBINDIR		=	$(SYSPREFIX)/usr$(LOCAL_PATH)/bin

OPT			=	-O
INC			=	./include
NODYN			=	-dn
LDFLAGS			=	-T 0x7fc000

# DEBUG			=	-g
DEBUG			=


# Common platform Defaults
STUBS	    = 
OFMT	    =
INSTALL	    =	/usr/ucb/install -c
# gcc:
# WARN	    =
WARN	    =	-v
MKDIR	    =   mkdir -p
TAGS        =   tags TAGS ETAGS

#
# OSR5 defines
#
OS	=	OSR5
OSR5_INSTALL=	/usr/bin/X11/scoinst -c
OSR5_LIBS=	-lnsl -lsocket 
OSR5_WARN=	-w3

#
# Solaris defines
#
OS	    =	SOLARIS
# gcc:
# SOL_CC	    =	gcc
# CC_SHARED   =	gcc -G
SOL_CC	    =	cc
CC_SHARED   =	cc -G
SOL_AR	    =	ar

SOL_LDFLAGS   = -Wl,-M,solaris.mapfile
SOL_LDSOFLAGS = -Wl,-M,solaris.mapfile.reduce -zdefs -ztext -h$@ -zcombreloc
SOL_LIBS      = -lmapmalloc -lc -lnsl -lsocket
SOL_LXLIB     =  liblxrun.so.1

SOL_PREFIX  =	/usr/local
SOL_BINDIR  =	$(SOL_PREFIX)/bin
SOL_MANDIR  =	$(SOL_PREFIX)/man
SOL_ETCDIR  =	$(SOL_PREFIX)/etc
SOL_LIBDIR  =	$(SOL_PREFIX)/lib
SOL_LXLDIR  =	$(SOL_PREFIX)/linux/lib


#
# UnixWare 7 defines
#
OS	=	UW7
UW_LIBS	=	./libsockstubs.a
UW_STUBS=	socket_stubs


#
# more common stuff
#
# gcc:
# PICFLAGS=	-fpic
PICFLAGS=	-Kpic
# OPTFLAGS=	
OPTFLAGS=	-DOLD_PROCFS

CFLAGS	=	$(OPT) $(DEBUG) -D$(OS) $(OFMT) $(WARN) $(PICFLAGS) -I$(INC) \
		-DDEF_LINUX_ROOT=\"$(REAL_LXROOT)\" -DEMULATOR=\"$(EMULATOR)\"\
		-DPATHMAPFILE -DLXRUN_AUTO_PATH_BEHAVIOR $(OPTFLAGS) \
		-DTRACE_DIR=\"$(TRACE_DIR)\" \
		-Dmalloc=lx_malloc -Drealloc=lx_realloc -Dfree=lx_free \
		-Dcalloc=lx_calloc -Dstrdup=lx_strdup

SO_OBJS	=	ld-linux.o
LX_OBJS	=	lxrun.o
AS_OBJS	=	lxstart.o alloca.o getput.o sendsig.o

COMMON_OBJS =		\
	$(AS_OBJS)	\
	admin.o		\
	devmap.o	\
	errno.o		\
	errors.o	\
	fileio.o	\
	filesys.o	\
	ioctl.o		\
	ipc.o		\
	loader.o	\
	lxruninit.o	\
	misc.o		\
	mmap.o		\
	modules.o	\
	msg.o		\
	pathname.o	\
	process.o	\
	procfs.o	\
	sem.o		\
	shm.o		\
	signal.o	\
	socket.o	\
	stat.o		\
	syscall.o	\
	sysent.o	\
	tioclinux.o     \
	trace.o		\
	traceargs.o	\
	verify.o	\
	version.o

.KEEP_STATE:

all: choose_platform

choose_platform:
	@case `uname -s` in						\
		SCO_SV)	$(MAKE) -f $(MAKEFILE)				\
				OS=OSR5					\
				OFMT=-belf                              \
                                INSTALL="$(OSR5_INSTALL)"		\
				LIBS="$(OSR5_LIBS)"			\
				WARN="$(OSR5_WARN)"			\
				osr5;					\
				;;					\
		UnixWare)  $(MAKE) -f $(MAKEFILE)			\
				OS=UW7					\
				LIBS="$(UW_LIBS)"			\
				STUBS="$(UW_STUBS)"			\
				uw7;					\
				;;					\
		UNIX_SV)  $(MAKE) -f $(MAKEFILE)			\
				OS=UNIXWARE				\
				CC="cc -DDSP_GETOPTR_HACK"		\
				LIBS="-lsocket -lc89"			\
				STUBS=""				\
				uw2;					\
				;;					\
		SunOS)  $(MAKE) -f $(MAKEFILE)				\
				OS=SOLARIS				\
				CC="$(SOL_CC)"				\
				LIBS="$(SOL_LIBS)"			\
				LDFLAGS="$(SOL_LDFLAGS)"		\
				AR="$(SOL_AR)"				\
				EMULATOR="/usr/bin/lxrun"		\
				sol;					\
				;;					\
		*)		echo Unknown Platform: `uname -s`	\
				;;					\
	esac

#force stand alone and uw7 build
sa_uw7: $(PROG) $(LXRUN_LOADER)
	$(MAKE) -f $(MAKEFILE)	OS=UW7					\
				LIBS="$(UW_LIBS)"			\
				STUBS=$(UW_STUBS)			\
                                LOCAL_PATH=/local                       \
				uw7

#force stand alone and osr5 build
sa_osr5: $(PROG)
	$(MAKE) -f $(MAKEFILE) 	OS=OSR5					\
				OFMT=-belf                              \
                                INSTALL="$(OSR5_INSTALL)"		\
				LIBS="$(OSR5_LIBS)"			\
				WARN=$(OSR5_WARN)			\
                                LOCAL_PATH=/local                       \
				osr5

sa_uw2: $(PROG)
	$(MAKE) -f $(MAKEFILE) 	OS=UNIXWARE				\
				CC="cc -DDSP_GETOPTR_HACK"		\
				LIBS="-lsocket -lc89"			\
				STUBS=""				\
                                LOCAL_PATH=/local                       \
				uw2

# default (standalone) build 
sa: $(PROG) $(LXRUN_LOADER)

uw7: $(PROG) $(LXRUN_LOADER)

uw2: $(PROG)

osr5: $(PROG)

sol: $(PROG) $(SOL_LXLIB)


$(PROG): $(LX_OBJS) $(COMMON_OBJS) $(STUBS)
	$(CC) $(CFLAGS) $(LDFLAGS) -o $(PROG) $(LX_OBJS) $(COMMON_OBJS) $(LIBS)

$(LXRUN_LOADER): $(SO_OBJS) $(COMMON_OBJS) $(STUBS)
	$(CC) $(NODYN) $(CFLAGS) $(LDFLAGS) -o $(LXRUN_LOADER) $(SO_OBJS) \
	    $(COMMON_OBJS) $(LIBS)

$(SOL_LXLIB): $(SO_OBJS) $(COMMON_OBJS) 
	@rm -f $(SOL_LXLIB)
	$(CC_SHARED) $(CFLAGS) $(SOL_LDSOFLAGS) -o $(SOL_LXLIB) $(SO_OBJS) \
		$(COMMON_OBJS) $(LIBS)
 
#
# build a stub sockets library so we can do a static link on UnixWare 7
#
socket_stubs:   libsockstubs.a

libsockstubs.a: sockstubs.o
	-rm -f libsockstubs.a
	$(AR) qv libsockstubs.a sockstubs.o

install: all
	@case `uname -s` in						\
		SCO_SV)	$(MAKE) -f $(MAKEFILE)				\
				OS=OSR5					\
                                INSTALL="$(OSR5_INSTALL)"		\
				LIBS="$(OSR5_LIBS)"			\
				WARN=$(OSR5_WARN)			\
				osr5install;				\
				;;					\
		UnixWare)  $(MAKE) -f $(MAKEFILE)			\
				OS=UW7					\
				LIBS="$(UW_LIBS)"			\
				STUBS="$(UW_STUBS)"			\
				uw7install;				\
				;;					\
		SunOS)  $(MAKE) -f $(MAKEFILE)				\
				OS=SOLARIS				\
				LIBS="$(SOL_LIBS)"			\
				LDFLAGS="$(SOL_LDFLAGS)"		\
				EMULATOR="/usr/bin/lxrun"		\
				solinstall;				\
				;;					\
		*)		echo Unknown Platform: `uname -s`	\
				;;					\
	esac

decoder:
	-[ -d $(DECODE_DIR) ] && { cd $(DECODE_DIR); make install; }

import_from_linux: decoder
	-[ -d $(LINUX_IMPORT_DIR) ] && { \
		cp $(LINUX_IMPORT_DIR)/$(NAME_SERVICE) . ; }
	-[ -d $(LINUX_IMPORT_DIR) ] && { \
	$(DECODE) -i $(LINUX_IMPORT_DIR)/$(NEW_LINUX_LOADER_ENC) ; }
	-[ -d $(LINUX_IMPORT_DIR) ] && { \
	$(DECODE) -i $(LINUX_IMPORT_DIR)/$(LINUX_LOADER_ENC) ; }
	-[ -d $(LINUX_IMPORT_DIR) ] && { \
	$(DECODE) -i $(LINUX_IMPORT_DIR)/$(CACHE_ENC) ; }
	-[ -d $(LINUX_IMPORT_DIR) ] && { \
	$(DECODE) -i $(LINUX_IMPORT_DIR)/$(LINUX_LOCALE_ENC) ; }
	-[ -d $(LINUX_IMPORT_DIR) ] && { \
	$(DECODE) -i $(LINUX_IMPORT_DIR)/$(LINUX_LIBS_TAR_ENC) ; }

osr5install: bin_install lenv_install linuxldr_links

uw7install: bin_install so_install lenv_install 

solinstall: 
	$(MKDIR) $(BINDIR)
	$(MKDIR) $(SOL_ETCDIR)
	$(MKDIR) $(SOL_LIBDIR)
	$(MKDIR) $(SOL_LXLDIR)
	$(MKDIR) $(SOL_MANDIR)
	$(MKDIR) $(SOL_MANDIR)/html
	$(MKDIR) $(SOL_MANDIR)/html/lxrun
	@echo "Installing ld-linux.so.1"
	@$(INSTALL) ld-linux.so.1 $(SOL_LIBDIR)
	@echo "Installing lxrun"
	@$(INSTALL) lxrun $(SOL_BINDIR)
	./SyscallScript
	@for i in doc/*; \
	do \
		echo "Installing $$i"; \
		$(INSTALL) $$i $(SOL_MANDIR)/html/lxrun; \
	done;

#install pathmap and executable and systemdir links
bin_install : dir_install
	$(INSTALL) $(PATHMAP) $(LXROOT)
	$(INSTALL) $(PROG) $(BINDIR)
	-strip $(BINDIR)/$(PROG)
	-mcs -d $(BINDIR)/$(PROG)
	-rm -f $(SYSBINDIR)/$(PROG)
	-ln -s $(BINDIR)/$(PROG) $(SYSBINDIR)

# install lxrun (faked loader) shared object
# link systempath linux loaders to lxrun shared object
so_install:
	[ -d $(LIBDIR) ]    || $(MKDIR) $(LIBDIR)
	[ -d $(SYSLIBDIR) ] || $(MKDIR) $(SYSLIBDIR)
	$(INSTALL) $(LXRUN_LOADER) $(LIBDIR)
	-rm -f $(SYSLIBDIR)/$(GENERIC_LINUX_LOADER)
	-rm -f $(SYSLIBDIR)/$(NEW_GENERIC_LINUX_LOADER)
	-ln -s $(LIBDIR)/$(LXRUN_LOADER) $(SYSLIBDIR)/$(GENERIC_LINUX_LOADER)
	-ln -s $(LIBDIR)/$(LXRUN_LOADER) $(SYSLIBDIR)/$(NEW_GENERIC_LINUX_LOADER)

# link systempath linux loaders to generic linux loaders in  LXROOT
# use so_install instead if lxrun.so available and platform can exec linux ELF
linuxldr_links: 
	-rm -f $(SYSLIBDIR)/$(GENERIC_LINUX_LOADER)
	-rm -f $(SYSLIBDIR)/$(NEW_GENERIC_LINUX_LOADER)
	-ln -s $(LIBDIR)/$(GENERIC_LINUX_LOADER) $(SYSLIBDIR)
	-ln -s $(LIBDIR)/$(NEW_GENERIC_LINUX_LOADER) $(SYSLIBDIR)


#setup lxroot and system directories
dir_install:
	[ -d $(BINDIR) ] || $(MKDIR) $(BINDIR)
	[ -d $(LIBDIR) ] || $(MKDIR) $(LIBDIR)
	[ -d $(SYSBINDIR) ] || $(MKDIR) $(SYSBINDIR)
	[ -d $(SYSLIBDIR) ] || $(MKDIR) $(SYSLIBDIR)


# install linux environment files
# link generic linux loaders to real files
lenv_install: import_from_linux  lenvdir_install
	-[ -f $(LINUX_LOADER) ] && $(INSTALL) $(LINUX_LOADER) $(LIBDIR)
	-[ -f $(NEW_LINUX_LOADER) ] && $(INSTALL) $(NEW_LINUX_LOADER) $(LIBDIR)
	-[ -f $(CACHE) ] && $(INSTALL) $(CACHE)        $(ETCDIR)
	-[ -f $(NAME_SERVICE) ] && $(INSTALL) $(NAME_SERVICE) $(ETCDIR)
	-[ -f $(LINUX_LOCALE_TAR_Z) ] && zcat $(LINUX_LOCALE_TAR_Z) | (cd $(LXROOT); tar xf -)
	-[ -f $(LINUX_LIBS_TAR_Z) ] && zcat $(LINUX_LIBS_TAR_Z) | (cd $(LXROOT); tar xf -)
	-[ -f $(LINUX_LOADER) ] && { \
	    rm -f $(LIBDIR)/$(GENERIC_LINUX_LOADER); \
	    ln -s $(LINUX_LOADER) $(LIBDIR)/$(GENERIC_LINUX_LOADER) ; }
	-[ -f $(NEW_LINUX_LOADER) ] && { \
	    rm -f $(LIBDIR)/$(NEW_GENERIC_LINUX_LOADER); \
	    ln -s $(NEW_LINUX_LOADER) $(LIBDIR)/$(NEW_GENERIC_LINUX_LOADER); }


#setup linux env directories
lenvdir_install:
	[ -d $(ETCDIR) ] || $(MKDIR) $(ETCDIR)
	[ -d $(LXROOT)/usr/lib ]       || $(MKDIR) $(LXROOT)/usr/lib	
	[ -d $(LXROOT)/usr/X11R6/bin ] || $(MKDIR) $(LXROOT)/usr/X11R6/bin
	[ -d $(LXROOT)/usr/X11R6/lib/X11/app-defaults ] || $(MKDIR) $(LXROOT)/usr/X11R6/lib/X11/app-defaults
	[ -d $(LXROOT)/usr/local/bin ] || $(MKDIR) $(LXROOT)/usr/local/bin
	[ -d $(LXROOT)/usr/local/lib ] || $(MKDIR) $(LXROOT)/usr/local/lib


clean:
	-rm -f $(OBJS) $(SO_OBJS) $(LX_OBJS) $(COMMON_OBJS) \
		sockstubs.o libsockstubs.a $(TAGS)
	([ -d $(DECODE_DIR) ] && { cd $(DECODE_DIR); make clean; }) || true

clobber: clean
	-rm -f $(PROG) $(LXRUN_LOADER) $(NAME_SERVICE) \
		$(NEW_LINUX_LOADER) $(LINUX_LOADER) $(CACHE) \
		$(LINUX_LOCALE_TAR_Z) $(LINUX_LIBS_TAR_Z)
	-[ -d $(DECODE_DIR) ] && { cd $(DECODE_DIR); make clobber; }

clean_imports:
	rm -f  $(NAME_SERVICE) \
		$(NEW_LINUX_LOADER) $(LINUX_LOADER) $(CACHE) \
		$(LINUX_LOCALE_TAR_Z) $(LINUX_LIBS_TAR_Z)

encodebin:  decoder
	cd $(LINUX_IMPORT_DIR); \
	   ../$(ENCODE) -i $(NEW_LINUX_LOADER) > $(NEW_LINUX_LOADER_ENC); \
	   ../$(ENCODE) -i $(LINUX_LOADER) > $(LINUX_LOADER_ENC); \
	   ../$(ENCODE) -i $(CACHE) > $(CACHE_ENC); \
	   ../$(ENCODE) -i $(LINUX_LOCALE_TAR_Z) > $(LINUX_LOCALE_ENC); \
	   ../$(ENCODE) -i $(LINUX_LIBS_TAR_Z) > $(LINUX_LIBS_TAR_ENC);

sockstubs.o:	lxrun.h			\
		trace.h			\
		$(MAKEFILE)

lxrun.o:	lxrun.c			\
		lxrun.h			\
		trace.h			\
		version.h		\
		$(MAKEFILE)

ld-linux.o:	ld-linux.c		\
		lxrun.h			\
		version.h		\
		auxv.h			\
		devmap.h		\
		$(INC)/linux/elf.h	\
		$(MAKEFILE)

loader.o:	loader.c		\
		lxrun.h			\
		trace.h			\
		auxv.h			\
		$(INC)/linux/a.out.h	\
		$(INC)/linux/elf.h	\
		$(MAKEFILE)

devmap.o:	devmap.c		\
		procfs.h		\
		devmap.h		\
		$(MAKEFILE)

syscall.o:	syscall.c		\
		lxrun.h			\
		trace.h			\
		sysent.h		\
		$(MAKEFILE)

getput.o:	getput.s		\
		$(MAKEFILE)

sysent.o:	sysent.c		\
		lxrun.h			\
		sysent.h		\
		syscalls.h		\
		$(MAKEFILE)

admin.o:	admin.c			\
		lxrun.h			\
		$(INC)/linux/utsname.h	\
		$(INC)/linux/sysinfo.h	\
		$(MAKEFILE)

errno.o:	errno.c			\
		lxrun.h			\
		$(INC)/linux/errno.h	\
		$(MAKEFILE)

fileio.o:	fileio.c		\
		lxrun.h			\
		devmap.h		\
		procfs.h		\
		trace.h			\
		$(INC)/linux/fcntl.h	\
		$(MAKEFILE)

filesys.o:	filesys.c		\
		lxrun.h			\
		$(INC)/linux/statfs.h	\
		$(MAKEFILE)

ioctl.o:	ioctl.c			\
		devmap.h		\
		lxrun.h			\
		trace.h			\
		$(INC)/linux/socket.h	\
		$(INC)/linux/termios.h	\
		$(MAKEFILE)

ipc.o:		ipc.c			\
		lxrun.h			\
		$(INC)/linux/ipc.h	\
		$(MAKEFILE)

lxruninit.o:	lxruninit.c		\
		trace.h			\
		devmap.h		\
		lxrun.h			\
		$(MAKEFILE)

misc.o:		misc.c			\
		lxrun.h			\
		trace.h			\
		$(MAKEFILE)

mmap.o:		mmap.c			\
		lxrun.h			\
		trace.h			\
		$(INC)/linux/types.h	\
		$(INC)/linux/mman.h	\
		$(MAKEFILE)

modules.o:	modules.c		\
		lxrun.h			\
		$(MAKEFILE)

msg.o:		msg.c			\
		lxrun.h			\
		$(INC)/linux/msg.h	\
		$(MAKEFILE)

pathname.o:	pathname.c		\
		lxrun.h			\
		trace.h			\
		$(MAKEFILE)

process.o:	process.c		\
		lxrun.h			\
		trace.h			\
		$(INC)/linux/types.h	\
		$(INC)/linux/resource.h	\
		$(MAKEFILE)

procfs.o:	procfs.c		\
		lxrun.h			\
		trace.h			\
		procfs.h		\
		$(MAKEFILE)

sem.o:		sem.c			\
		lxrun.h			\
		$(INC)/linux/sem.h	\
		$(MAKEFILE)

sendsig.o:	sendsig.s		\
		$(MAKEFILE)

shm.o:		shm.c			\
		lxrun.h			\
		$(INC)/linux/shm.h	\
		$(MAKEFILE)

signal.o:	signal.c		\
		lxrun.h			\
		trace.h			\
		$(INC)/linux/resource.h	\
		$(INC)/linux/signal.h	\
		$(INC)/linux/wait.h	\
		$(INC)/linux/sigcontext.h	\
		$(MAKEFILE)

socket.o:	socket.c		\
		lxrun.h			\
		trace.h			\
		$(INC)/linux/socket.h	\
		$(MAKEFILE)

stat.o:		stat.c			\
		lxrun.h			\
		procfs.h		\
		$(INC)/linux/stat.h	\
		$(MAKEFILE)

verify.o:	verify.c		\
		lxrun.h			\
		$(MAKEFILE)

errors.o:	errors.c		\
		lxrun.h			\
		$(MAKEFILE)

tioclinux.o:    tioclinux.c             \
		lxrun.h			\
		$(MAKEFILE)

trace.o:	trace.c			\
		trace.h			\
		lxrun.h			\
		sysent.h		\
		syscalls.h		\
		$(INC)/linux/ipc.h	\
		$(INC)/linux/socket.h	\
		$(INC)/linux/sigcontext.h\
		$(MAKEFILE)

version.o:	version.c		\
		version.h		\
		$(MAKEFILE)

traceargs.o:	traceargs.c		\
		lxrun.h			\
		sysent.h		\
		trace.h			\
		$(INC)/linux/ipc.h	\
		$(INC)/linux/socket.h	\
		$(INC)/linux/fcntl.h	\
		$(INC)/linux/signal.h	\
		$(INC)/linux/mman.h	\
		$(MAKEFILE)

syscalls:
	./SyscallScript


tags:
	ctags *.[ch]

TAGS:
	etags *.[ch]

ETAGS:	TAGS
	cp TAGS ETAGS

# Solaris cscope support

CSDIRS=		.
CSINCS=		$(CSDIRS:%=-I%)

CSCOPE=		cscope-fast
CSFLAGS=	-b

.PRECIOUS:	cscope.out

cscope.out:	cscope.files FRC
		$(CSCOPE) $(CSFLAGS)

cscope.files:	FRC
	@ -$(RM) cscope.files
	echo "$(CSINCS)" > cscope.files
	-find $(CSDIRS) -name SCCS -prune -o \
		-name '*.[Ccsh]' -print >> cscope.files

cscope.clean:
	-$(RM) cscope.files cscope.out

FRC:

