#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% %Q% SMI"
#endif
/*
 * Copyright (C) 1996-1997 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#ifdef UW7
#   define _EFTSAFE
#endif
#include	<sys/socket.h>
#include	<netdb.h>

#include	"lxrun.h"
#include	"devmap.h"
#include	"trace.h"

char	*linux_root	= NULL;
char	*linux_uname	= NULL;

extern char emulator[];

#ifdef PATHMAPFILE
#include <malloc.h>

extern int parse_pathmap_table(const char *pathmapfn);
#endif

/*
** Perform initializations common to lxrun.c and ld-linux.c
*/

void
lxrun_init(void)
{
	char		*display;
	/* Must be static to make putenv() happy. */
	static char	displaybuf[64];

	TRACE_INIT(getenv("LXRUN_TRACE"));

	TRACE_MSG(TRACE_ALL, ("TRACE: emulator = %s\n", emulator));

	TRACE_MSG(TRACE_ALL, ("TRACE: pid = %d, ppid = %d\n",
				getpid(), getppid()));

        if (!linux_root)
        {
	    linux_root = getenv("LINUX_ROOT");
#ifdef DEF_LINUX_ROOT
	    if (linux_root == NULL)
		linux_root = DEF_LINUX_ROOT;
#endif
        }
        TRACE_MSG(TRACE_ALL, ("TRACE: linux_root = %s\n", linux_root));

        if (!linux_uname)
	    linux_uname = getenv("LINUX_UNAME");

        TRACE_MSG(TRACE_ALL, ("TRACE: linux_uname = %s\n",
			      linux_uname == NULL ? "(unset)": linux_uname));

#ifndef NO_DISPLAY_HACK
	/*
	** For local displays, OpenServer uses ":0.0" and UnixWare uses
	** "unix:0.0".  The former causes Linux X11 binaries to try to
	** use shared memory, which won't work with the native X server.
	** The latter causes a name lookup for the machine "unix" which
	** probably fails.  We'll detect these two cases and set DISPLAY
	** to contain a valid machine name instead.
	** Note that this causes a loss in performance when running X apps
	** since we have to go through TCP.
	*/
	if ((display = getenv("DISPLAY")) != NULL &&
#if defined(UNIXWARE) || defined(UW7)
		/* These platforms use "unix:0.0" for local X */
		(*((long *)display)==*((long *)"unix") && display[4]==':')
#else
		/* Other platforms use ":0.0" */
		(display[0] == ':')
#endif
			) {

#if defined(UNIXWARE) || defined(UW7)
		display+=4;		/* skip "unix" */
#endif

		strncpy(displaybuf, "DISPLAY=127.0.0.1", sizeof(displaybuf));
		strncat(displaybuf, display, sizeof(displaybuf));
		
		if (putenv(displaybuf)) {
			fprintf(stderr, "putenv(%s) failed\n", displaybuf);
			exit(1);
		}
	}
#endif  /* NO_DISPLAY_HACK */

        TRACE_MSG(TRACE_ALL, ("TRACE: display = %s\n",
		(getenv("DISPLAY")) ? getenv("DISPLAY") : "(unset)"));

	if (init_signals() != 0)
		fatal(errno, "can't set up signal handlers");

	devmap_init();

#ifdef PATHMAPFILE
        {
            char	*pathmapfn;
    	    char	*pathmapdef = NULL;

	    if(!(pathmapfn = getenv("LINUX_PATHMAP")))
	    {
		/* Assume $LINUX_ROOT/PATHMAP */
	        if (!(pathmapdef = (char *)malloc(strlen(linux_root)+9)))
	            fatal(ENOMEM, "Cannot malloc for pathmapdef\n");

#if defined(SOLARIS)
		/*++ change this so that if deffed fm Makefile that dfn used */
		pathmapdef = strdup("/usr/local/lxrun/PATHMAP");
#else
		strcpy(pathmapdef, linux_root);
		strcat(pathmapdef, "/PATHMAP");
#endif
		pathmapfn = pathmapdef;
	    }
	    if(parse_pathmap_table(pathmapfn) < 0)
		fatal(errno, "Can't open pathmap file \"%s\"",
					pathmapfn);
	    if(pathmapdef)
		free(pathmapdef);
	}
#endif	/* PATHMAPFILE */
}
