/
/ Copyright (C) 1997 Michael Davidson.
/ All rights reserved.
/
/ This software is provided "as is" without express or implied warranty.
/
/ ident       "%Z%%M% %I%     %E% SMI"

	.text
/
/ lxstart(start, char **argv, char **envp, auxv_t *auxv)
/
	.globl	lxstart

START	=	8
ARGV	=	12
ENVP	=	16
AUXV	=	20

/
/ since lxstart never returns we don't need to save any
/ registers, but it's convenient to use %ebp as a frame
/ pointer so that we can get at the arguments to lxstart()
/ easily while building the stack frame for the program
/
/ (there's no need to save %ebp either, but having the
/ function arguments start at their "normal" offset from
/ %ebp, (ie 8), seems like the "right" thing to do ...)
/
	.align	4
lxstart:
	pushl	%ebp
	movl	%esp,%ebp
/
/ copy aux vector if it exists
/
	movl	AUXV(%ebp),%esi		/ pointer to aux vector
	testl	%esi,%esi
	je	copy_env		/ no aux vector
	xorl	%ecx,%ecx
	.align	4
next_aux:				/ count the number of aux entries
	movl	(%esi),%eax
	leal	8(%esi),%esi
	incl	%ecx
	testl	%eax,%eax
	jne	next_aux
	leal	(,%ecx,8),%eax		/ get byte count in eax
	movl	AUXV(%ebp),%esi		/ reload pointer to aux vector
	leal	(,%ecx,2),%ecx		/ get dword count in ecx
	subl	%eax,%esp		/ make space on stack
	movl	%esp,%edi
	rep
	movsl				/ copy aux vector

	.align	4
/
/ copy env vector
/
copy_env:
	movl	ENVP(%ebp),%esi		/ pointer to env vector
	xorl	%ecx,%ecx
	.align	4
next_env:				/ count the number of env entries
	movl	(%esi),%eax
	leal	4(%esi),%esi
	incl	%ecx
	testl	%eax,%eax
	jne	next_env
	leal	(,%ecx,4),%eax		/ get byte count in eax
	movl	ENVP(%ebp),%esi		/ reload pointer to env vector
	subl	%eax,%esp		/ make space on stack
	movl	%esp,%edx		/ save new envp in edx
	movl	%esp,%edi
	rep
	movsl				/ copy env vector
/
/ copy arg vector
/
	movl	ARGV(%ebp),%esi		/ pointer to arg vector
	xorl	%ecx,%ecx
	.align	4
next_arg:				/ count the number of arg entries
	movl	(%esi),%eax
	leal	4(%esi),%esi
	incl	%ecx
	testl	%eax,%eax
	jne	next_arg
	leal	(,%ecx,4),%eax		/ get byte count in eax
	movl	ARGV(%ebp),%esi		/ reload pointer to arg vector
	subl	%eax,%esp		/ make space on stack
	movl	%esp,%edi
	rep
	movsl				/ copy arg vector
/
/ now set up the final part of the stack like this
/
/	envp			(COFF only)
/	argv			(COFF only)
/	argc
/	start address
/
/ Linux COFF needs both envp and argv on the stack as well as argc
/ Linux ELF just needs argc, and the program startup code sets up
/ argv and envp itself
/
/ We use the presence of an aux vector to indicate ELF layout
/ (this means that static ELF must have an aux vector even though
/ it doesn't actually need one)
/
	movl	AUXV(%ebp),%ebx		/ get ELF / COFF flag
	shrl	$2,%eax			/ get arg count in eax
	decl	%eax			/ argc doesn't include the NULL arg
	movl	START(%ebp),%ecx	/ ecx = start
	testl	%ebx,%ebx
	jne	is_elf
	movl	%esp,%ebx		/ ebx = argv

	pushl	%edx			/ envp
	pushl	%ebx			/ argv
is_elf:
	pushl	%eax			/ argc
	pushl	%ecx			/ start address

/
/ clear registers - mostly to assist in debugging
/
	xorl	%eax,%eax
	xorl	%ebx,%ebx
	xorl	%ecx,%ecx
	xorl	%edx,%edx
	xorl	%esi,%esi
	xorl	%edi,%edi
	xorl	%ebp,%ebp

	ret				/ "return" to start of loaded program


