#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% %Q% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<sys/types.h>

#include	<sys/sysi86.h>

#if /* !defined(SOLARIS) || */ !defined(V86SC_IOPL)
/* Solaris 7 barfs on this #include; Solaris 2.6 requires it.
** The above is a temporary hack.  Eventually, we should be detecting
** whether we're using 2.6 or 7 and act accordingly.  */
# include	<sys/v86.h>
#endif

#include	<sys/time.h>
#include	<sys/times.h>
#include	<sys/select.h>

#include	"lxrun.h"
#include	"trace.h"

int sysi86(int cmd, ...);

/*
 * assorted random system calls
 */

/*
 *	ioperm		(PARTLY IMPLEMENTED)
 *	iopl		(OSR5 only)
 *	vhangup		(NOT IMPLEMENTED)
 *	idle		(NOT IMPLEMENTED)
 *	vm86		(NOT IMPLEMENTED)
 *	modify_ldt	(NOT IMPLEMENTED)
 *	clone		(NOT IMPLEMENTED)
 *	sysctl		(NOT IMPLEMENTED)
 *	setitimer
 *	getitimer
 *	nanosleep	(PARTLY IMPLEMENTED) (direct-mapped on Solaris)
 */

int
lx_ioperm(unsigned long fron, unsigned long num, int turn_on)
{
#if defined(OSR5) || defined(SOLARIS) 
	return sysi86(SI86V86, V86SC_IOPL, turn_on ? 0x3000 : 0);
#elif defined(UNIXWARE) || defined(UW7)
	/* Untested.  Is this correct?  */
	return sysi86(SI86IOPL, turn_on ? 3 : 0);
#else
	return enosys("ioperm");

#endif
}

int
lx_iopl(int level)
{
#if defined(OSR5)
	if (level > 3 || level < 0) {
	    return enoany(EINVAL, "iopl() - ! level <0 or >3");
	}

	return sysi86(SI86V86, V86SC_IOPL, level << 24);
#elif defined(SOLARIS)
	/* XXX: untested */
	if (level > 3 || level < 0) {
	    return enoany(EINVAL, "iopl() - level <0 or >3");
	}

	return sysi86(SI86V86, V86SC_IOPL, level << 12);
#else
	return enosys("iopl");
#endif
}


#ifdef UW7
/* 
 * WARNING, WARNING, WARNING!!!!
 *
 * I hate to do this, but real itimers are stepping on memory (UW7),
 * and until I can figure it out, this dummy routine allows
 * Communicator and StarOffice 4.0 to work.
 */
int
lx_setitimer(int which, struct itimerval *value, struct itimerval *ovalue)
{
	if (which)
		return setitimer(which, value, ovalue);
	else
//		return setitimer(which, value, ovalue);
		return 0;
}	

int
lx_getitimer(int which, struct itimerval *value)
{
	return getitimer(which, value);
}
#endif	/* UW7 */


#ifndef SOLARIS 	/* until find equiv on other platforms */
/* int lx_nanosleep()	{ return enosys("nanosleep"); } */

/* Let's fake-out a nanosleep using other delay tactics.
** BUG: The time returned in "rem" is a reasonable guess, but probably
**	not accurate.
** BUG: We're limited to microsecond accuracy, but then again, so is the
**	Linux nanosleep() implementation.
*/

struct lx_timespec {
	time_t  tv_sec;		/* seconds */
	long    tv_nsec;	/* ns (1,000,000,000 per sec) */
};

int lx_nanosleep(const struct lx_timespec *req, struct lx_timespec *rem)
{
	struct timeval t;

	/* Do range checking the same way a real nanosleep() would. */
	if(!req)
		return enoany(EINVAL, "nanosleep() - req is NULL");
	if(req->tv_nsec < 0 || req->tv_nsec > 999999999)
		return enoany(EINVAL, "nanosleep() - tv_nsec out of range");
	if(req->tv_sec < 0)
		return enoany(EINVAL, "nanosleep() - tv_sec negative");

	/* Since the Linux nanosleep() is actually only accurate to a
	** 1us granularity at best, we don't lose any real precision
	** by using select. */
	t.tv_sec = req->tv_sec;
	t.tv_usec = req->tv_nsec/1000;
	if(select(0, 0, 0, 0, &t) == -1)
		{
		if(errno == EINTR)
			{
			/* We were interrupted, but select() doesn't tell
			** us how much time was remaining on the clock so
			** in the true style of the great hack, we fake it. */
			if(rem)
				{
				/* Estimate the remaining time as half the
				** original time, so if we're repeatedly
				** interrupted, we never finish.  :-)  */
				rem->tv_sec = req->tv_sec / 2;
				rem->tv_nsec = req->tv_nsec / 2;
				return enoany(EINTR,
						"nanosleep() - interrupted");
				}
			}
		return -1;
		}
	return 0;
}
#endif

int lx_vhangup()	{ return enosys("vhangup"); }
int lx_idle()		{ return enosys("idle"); }
int lx_vm86()		{ return enosys("vm86"); }
int lx_modify_ldt()	{ return enosys("modify_ldt"); }
int lx_clone()		{ return enosys("clone"); }
int lx_sysctl()		{ return enosys("sysctl"); }

/*
 * setup() is only called at boot, early on in /sbin/init.
 * acct() is for process accounting.
 * ustat() is some wacky pre-statfs() statfs().
 * 
 * The rest aren't even implemented in Linux.
 */
int lx_acct()		{ return enosys("acct"); }
int lx_afs_syscall()	{ return enosys("afs_syscall"); }
int lx_break()		{ return enosys("break"); }
int lx_ftime()		{ return enosys("ftime"); }
int lx_gtty()		{ return enosys("gtty"); }
int lx_lock()		{ return enosys("lock"); }
int lx_mpx()		{ return enosys("mpx"); }
int lx_phys()		{ return enosys("phys"); }
int lx_prof()		{ return enosys("prof"); }
int lx_profil()		{ return enosys("profil"); }
int lx_setup()		{ return enosys("setup"); }
int lx_stty()		{ return enosys("stty"); }
int lx_ulimit()		{ return enosys("ulimit"); }
int lx_ustat()		{ return enosys("ustat"); }
