#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z% %M% %I% %D% %Q% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<stdio.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/mman.h>
#ifdef SOLARIS
#include	<sys/param.h>
#endif
#include	"lxrun.h"
#include	"trace.h"
#include	"linux/types.h"
#include	"linux/mman.h"

#ifndef	MAP_FAILED
#define	MAP_FAILED	(-1)
#endif

/*
 * mmap() related system calls
 */

/*
 *	mmap
 *	munmap
 *	mprotect
 *	msync
 *	mlock		(NOT IMPLEMENTED)
 *	munlock		(NOT IMPLEMENTED)
 *	mlockall	(NOT IMPLEMENTED)
 *	munlockall	(NOT IMPLEMENTED)
 *	mremap
 */

static int dzfd = -1;



void *
dz_mmap(void *addr, size_t len, int prot, int flags, off_t off)
{
	if (dzfd == -1)
        {
		dzfd = open("/dev/zero", O_RDWR);
		fcntl(dzfd, F_SETFD, FD_CLOEXEC);
	}
	if (dzfd == -1)
		return (void*)MAP_FAILED;

	return mmap(addr, len, prot, flags, dzfd, off);
}

/* Possibly invalidate the /dev/zero fd
 * (I don't see how this is going to work since close won't ever have
 *  the fd that dz_mmap opened /dev/zero with  - so we just hold
 * that fd for future use ) 
 * The fear is that the app might close the cached dzfd. Some shells,
 * for example, close all their descriptors on startup. If they do that,
 * then either the above mmap will fail (EBADFD), or you'll start mmaping
 * bits of some other file. Since we're interposing on close anyway, close
 * always calls dz_invalidate to see if the app is doing this.
 */
void
dz_invalidate(int fd)
{
    if (fd == dzfd)
	dzfd = -1;
}



void *
lx_mmap(unsigned long *av)
{
	void	*addr;
	size_t	len;
	int	lx_prot;
	int	lx_flags;
	int	fd;
	off_t	off;
	int	prot	= 0;
	int	flags	= 0;
	void	*r;

	if (! verify_read(av, 6 * sizeof(av[0])))
		return (void *)MAP_FAILED;

	addr	= (void *)av[0];
	len	= (size_t)av[1];
	lx_prot	= (int)av[2];
	lx_flags= (int)av[3];
	fd	= (int)av[4];
	off	= (off_t)av[5];

	MMAP_TRACE_MSG(("TRACE: lx_mmap(0x%x, %d, %d, 0x%x, %d, %d)\n",
			addr, len, lx_prot, lx_flags, fd, off));

	if (lx_prot & LX_PROT_READ)
		prot |= PROT_READ;
	if (lx_prot & LX_PROT_WRITE)
		prot |= PROT_WRITE;
	if (lx_prot & LX_PROT_EXEC)
		prot |= PROT_EXEC;

	if (lx_flags & LX_MAP_SHARED)
		flags |= MAP_SHARED;
	if (lx_flags & LX_MAP_PRIVATE)
		flags |= MAP_PRIVATE;
	if (lx_flags & LX_MAP_FIXED)
		flags |= MAP_FIXED;
	
	if (lx_flags & LX_MAP_ANONYMOUS)
		return dz_mmap(addr, len, prot, flags, off);

	return mmap(addr, len, prot, flags, fd, off);
}	

int
lx_munmap(void *addr, size_t len)
{
	MMAP_TRACE_MSG(("TRACE: lx_munmap(0x%x, %d)\n",
			 addr, len));

	return munmap(addr, len);
}	

int
lx_mprotect(void *addr, size_t len, int lx_prot)
{
	int	prot	= 0;

	if (lx_prot & LX_PROT_READ)
		prot |= PROT_READ;
	if (lx_prot & LX_PROT_WRITE)
		prot |= PROT_WRITE;
	if (lx_prot & LX_PROT_EXEC)
		prot |= PROT_EXEC;

#ifdef OSR5
        if(mprotect(addr, len, (prot==PROT_NONE)
                                        ? PROT_READ : PROT_NONE) < 0 &&
                                        errno != ENOMEM)
            return (int)MAP_FAILED;
#endif

	return mprotect(addr, len, prot);
}


int lx_msync(void *start, size_t length, int lx_flags)
{
	int flags = 0;

	if(lx_flags & LX_MS_ASYNC)
		flags |= MS_ASYNC;
	if(lx_flags & LX_MS_INVALIDATE)
		flags |= MS_INVALIDATE;
	if(lx_flags & LX_MS_SYNC)
		flags |= MS_SYNC;

	return msync(start, length, flags);
}


int lx_mlock()		{ return enosys("mlock"); }
int lx_munlock()	{ return enosys("munlock"); }
int lx_mlockall()	{ return enosys("mlockall"); }
int lx_munlockall()	{ return enosys("munlockall"); }

void *
lx_mremap(void *oaddr, size_t old_size, size_t new_size, unsigned long flags)
{
	if (new_size < old_size) {
		uintptr_t oend = (uintptr_t)oaddr + PAGE_ALIGN(old_size);
		uintptr_t nend = (uintptr_t)oaddr + PAGE_ALIGN(new_size);
		if (munmap((void *)nend, oend - nend) == (int)MAP_FAILED)
                {
                    MMAP_TRACE_MSG(("TRACE: lx_mremap munmap(0x%x, %d)"
                                    " FAILED errno %d \n",
                                    nend, oend - nend, errno));
		}
		return oaddr;
	} else {
            /* if MREMAP_MAYMOVE set -  unmap and remap ?? */
                MMAP_TRACE_MSG(("MMAP : Can't lx_mremap(map growing) "
		    "(oaddr = %p, osize = %x, nsize = %x, flags = %x)\n",
		    oaddr, old_size, new_size, flags));
                errno = ENOMEM;         /* just fail it */
                return (void*)MAP_FAILED;
	}
}
