#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z%%M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include        <errno.h>
#include        <string.h>
#include        <sys/time.h>

#ifdef UW7
#   include     <sys/syscall.h>
#   define      _INKERNEL
#   include     <sys/socket.h>
#   undef SOCKETSYS
#   define SOCKETSYS    140
#   define SO_SOCKPAIR  27
int socket(int domain, int type, int protocol);
int setsockopt(int s, int level, int optname, const void *optval,
                size_t optlen);
int getsockopt(int s, int level, int optname, void *optval, size_t *optlen);
int bind(int socket, const struct sockaddr *name, size_t namelen);
int connect(int socket, const struct sockaddr *address, size_t address_len);
int getsockname(int socket, struct sockaddr *name, size_t *namelen);
int getpeername(int socket, struct sockaddr *name, size_t *namelen);
int socketpair(int d, int type, int protocol, int sv[2]);
ssize_t send(int s, const void *msg, size_t len, int flags);
ssize_t recv(int socket, void *buf, size_t len, int flags);
int accept(int s, struct sockaddr *addr, size_t *addrlen);
int shutdown(int socket, int how);
int listen(int socket, int backlog);
ssize_t sendto(int s, const void *msg, size_t len, int flags,
                const struct sockaddr *to, size_t tolen);
ssize_t recvfrom(int socket, void *buf, size_t len, int flags,
                struct sockaddr *from, size_t *fromlen);
#else
#   include     <sys/socket.h>
#endif

#include        <netinet/in.h>
#include        <stdio.h>
#include        <strings.h>

#include        "lxrun.h"
#include        "linux/socket.h"
#include        "trace.h"

    typedef struct
    {
        int             value;
        const char      *name;
    } s_name_t;

    static const char *lx_sockdomstr(int dom);
    static const char *ntv_sockdomstr(int dom);
    static const char *lx_socktypestr(int type);
    static const char *ntv_socktypestr(int type);
    static const char *lx_sockprotostr(int proto);
    static const char *ntv_sockprotostr(int proto);
    static const char *lx_sockoptstr(int opt);
    static const char *ntv_sockoptstr(int opt);
    static const char *lx_socklvlstr(int level);
    static const char *ntv_socklvlstr(int level);
    static const char *lx_sockcallstr(int call);
    static const char *strname(int value, const s_name_t *np);
    static const char *fdsetstr(int nfd, fd_set *fds);

static int xlate_sockopt(int optname);
static int xlate_socklvl(int level);

/*
 * OpenServer 5.0 socket function prototypes use int instead of size_t
 */
#if defined(OSR5)
#define SIZE_T  int
#else
#define SIZE_T  size_t
#endif

/*
 * sockets
 */
int
lx_old_select(unsigned long *av)
{
    int                 nfd;
    fd_set              *rfd;
    fd_set              *wfd;
    fd_set              *xfd;
    struct timeval      *tv;

    if (!verify_read(av, 5 * sizeof(*av)))
                return -1;

    nfd = (int)av[0];
    rfd = (fd_set *)av[1];
    wfd = (fd_set *)av[2];
    xfd = (fd_set *)av[3];
    tv = (struct timeval *)av[4];

    SELECT_TRACE_MSG((
            "TRACE: old_select(nfd %d, rfd %s, wfd %s, xfd %s, tv %g)\n",
                            nfd,
                            fdsetstr(nfd, rfd),
                            fdsetstr(nfd, wfd),
                            fdsetstr(nfd, xfd),
			    (tv != NULL) ?
                              ((double)tv->tv_sec +
			      ((double)tv->tv_usec / 1e6)) : 0.0));

    return select(nfd, rfd, wfd, xfd, tv);
}

int
lx_select(int nfd, fd_set *rfd, fd_set *wfd, fd_set *xfd, struct timeval *tv)
{
    SELECT_TRACE_MSG((
            "TRACE: select(nfd %d, rfd %s, wfd %s, xfd %s, tv %g)\n",
                            nfd,
                            fdsetstr(nfd, rfd),
                            fdsetstr(nfd, wfd),
                            fdsetstr(nfd, xfd),
			    (tv != NULL) ?
                              ((double)tv->tv_sec +
			      ((double)tv->tv_usec / 1e6)) : 0.0));

    return select(nfd, rfd, wfd, xfd, tv);
}

int lx_syslog()		{ return enosys("syslog"); }

int
lx_socket(int lx_domain, int lx_type, int lx_protocol)
{
    int         ntv_domain;
    int         ntv_type;
    int         ntv_protocol;


        ntv_domain =  ntv_type =  ntv_protocol = -1;
        switch (lx_domain)
        {
                case LX_AF_UNSPEC:      ntv_domain = AF_UNSPEC; break;
                case LX_AF_UNIX:        ntv_domain = AF_UNIX;   break;
                case LX_AF_INET:        ntv_domain = AF_INET;   break;
                case LX_AF_AX25:
                case LX_AF_IPX:
                default:
		    return enoany(EINVAL, "lx_socket() - bad lx_domain value");
        }

        switch (lx_type)
        {
                case LX_SOCK_STREAM:    ntv_type = SOCK_STREAM; break;
                case LX_SOCK_DGRAM:     ntv_type = SOCK_DGRAM;  break;
                case LX_SOCK_RAW:       ntv_type = SOCK_RAW;    break;
                case LX_SOCK_RDM:       ntv_type = SOCK_RDM;    break;
                case LX_SOCK_SEQPACKET: ntv_type = SOCK_SEQPACKET;      break;
                default:
		    return enoany(EINVAL, "lx_socket() - bad lx_type value");
        }

        switch (lx_protocol)
        {
                case LX_PF_UNSPEC:      ntv_protocol = PF_UNSPEC; break;
                case LX_PF_UNIX:        ntv_protocol = PF_UNIX; break;
                case LX_PF_INET:        ntv_protocol = PF_INET; break;
                case LX_PF_AX25:
                case LX_PF_IPX:
            default:
                ntv_protocol = lx_protocol;
                break;
        }

    SOCK_TRACE_MSG((
            "TRACE: socket(domain %s => %s, type %s => %s, proto %s => %s)\n",
                                        lx_sockdomstr(lx_domain),
                                        ntv_sockdomstr(ntv_domain),
                                        lx_socktypestr(lx_type),
                                        ntv_socktypestr(ntv_type),
                                        lx_sockprotostr(lx_protocol),
                                        ntv_sockprotostr(ntv_protocol)));

    if ((ntv_domain == -1) || (ntv_type == -1) || (ntv_protocol == -1))
    {
	    return enoany(EINVAL,
			  "lx_socket() - unset ntv_{domain,type,protocol}");
    }

    return socket(ntv_domain, ntv_type, ntv_protocol);
}

static int
xlate_sockopt(int optname)
{
    switch (optname)
    {
        case LX_SO_DEBUG:
            return SO_DEBUG;

        case LX_SO_REUSEADDR:
            return SO_REUSEADDR;

        case LX_SO_DONTROUTE:
            return SO_DONTROUTE;

        case LX_SO_BROADCAST:
            return SO_BROADCAST;

        case LX_SO_KEEPALIVE:
            return SO_KEEPALIVE;

        case LX_SO_OOBINLINE:
            return SO_OOBINLINE;

        case LX_SO_LINGER:
            return SO_LINGER;

        case LX_SO_SNDBUF:
            return SO_SNDBUF;

        case LX_SO_RCVBUF:
            return SO_RCVBUF;
    }

    return -1;
}

static int
xlate_socklvl(int level)
{
    switch (level)
    {
        case LX_SOL_SOCKET:
            return SOL_SOCKET;

        case IPPROTO_IP:
            return IPPROTO_IP;

#if IPPROTO_ICMP != LX_SOL_SOCKET
        case IPPROTO_ICMP:
            return IPPROTO_ICMP;
#endif

        case IPPROTO_IGMP:
            return IPPROTO_IGMP;

#ifdef IPPROTO_IPIP
        case IPPROTO_IPIP:
            return IPPROTO_IPIP;
#endif

        case IPPROTO_TCP:
            return IPPROTO_TCP;

        case IPPROTO_EGP:
            return IPPROTO_EGP;

        case IPPROTO_PUP:
            return IPPROTO_PUP;

        case IPPROTO_UDP:
            return IPPROTO_UDP;

        case IPPROTO_IDP:
            return IPPROTO_IDP;

        case IPPROTO_RAW:
            return IPPROTO_RAW;
    }

    return -1;
}

int
lx_setsockopt(int s, int level, int optname, void *optval, SIZE_T optlen)
{
    int ntv_opt   = xlate_sockopt(optname);
    int ntv_level = xlate_socklvl(level);

    SOCK_TRACE_MSG(("TRACE: setsockopt(opt %s => %s, level %s => %s)\n",
                                        lx_sockoptstr(optname),
                                        ntv_sockoptstr(ntv_opt),
					lx_socklvlstr(level),
					ntv_socklvlstr(ntv_level)));

    if ((ntv_opt == -1) || (ntv_level == -1))
    {
	return enoany(EINVAL, "lx_setsockopt() - ntv_{opt,level} unset");
    }

    return setsockopt(s, ntv_level, ntv_opt, optval, optlen);
}


int
lx_getsockopt(int s, int level, int optname, void *optval, SIZE_T *optlen)
{
    int ntv_opt = xlate_sockopt(optname);
    int ntv_level = xlate_socklvl(level);

    SOCK_TRACE_MSG(("TRACE: getsockopt(opt %s => %s, level %s => %s)\n",
					lx_sockoptstr(optname),
					ntv_sockoptstr(ntv_opt),
					lx_socklvlstr(level),
					ntv_socklvlstr(ntv_level)));

    if ((ntv_opt == -1) || (ntv_level == -1))
    {
	return enoany(EINVAL, "lx_getsockopt() - ntv_{opt,level} unset");
    }

    return getsockopt(s, ntv_level, ntv_opt, optval, optlen);
}

int
lx_socketcall(int call, unsigned long *av)
{
	register int	s;

	SOCK_TRACE_MSG(("TRACE: socketcall(call %s)\n", lx_sockcallstr(call)));

	s = (int)av[0];
	switch (call) 
	{
		case LX_SYS_SOCKET:
		    return lx_socket(s, (int)av[1], (int)av[2]);

		case LX_SYS_BIND:
		    return bind(s, (struct sockaddr *)av[1], (int)av[2]);

		case LX_SYS_CONNECT:
		    return connect(s, (struct sockaddr *)av[1], (SIZE_T)av[2]);

		case LX_SYS_LISTEN:
		    return listen(s, (int)av[1]);

		case LX_SYS_ACCEPT:
		    return accept(s,(struct sockaddr *)av[1], (SIZE_T *)av[2]);

		case LX_SYS_GETSOCKNAME:
		    return getsockname(s, (struct sockaddr *)av[1],
					(SIZE_T *)av[2]);

		case LX_SYS_GETPEERNAME:
		    return getpeername(s, (struct sockaddr *)av[1],
					(SIZE_T *)av[2]);

		case LX_SYS_SOCKETPAIR:
		    return socketpair(s, (int)av[1], (int)av[2], (int *)av[3]);

		case LX_SYS_SEND:
		    return send(s, (void *)av[1], (SIZE_T)av[2], (int)av[3]);

		case LX_SYS_SENDTO:
		    return sendto(s, (void *)av[1], (SIZE_T)av[2],
			(int)av[3], (struct sockaddr *)av[4], (SIZE_T)av[5]);

		case LX_SYS_RECV:
		    return recv(s, (void *)av[1], (SIZE_T)av[2], (int)av[3]);

		case LX_SYS_RECVFROM:
		    return recvfrom(s, (void *)av[1],
				(SIZE_T)av[2], (int)av[3],
				(struct sockaddr *)av[4], (SIZE_T *)av[5]);

		case LX_SYS_SHUTDOWN:
		    return shutdown(s, (int)av[1]);

		case LX_SYS_SETSOCKOPT:
		    return lx_setsockopt(s, (int)av[1], (int)av[2],
				(void *)av[3], (SIZE_T)av[4]);

		case LX_SYS_GETSOCKOPT:
		    return lx_getsockopt(s, (int)av[1], (int)av[2],
				(void *)av[3], (SIZE_T *)av[4]);
	}

	return enoany(EINVAL, "lx_socketcall() - unknown call");
}


#define S_NM(x) 	{ x, #x }
#define S_NM_END	{ NULL }

static const char *
lx_sockdomstr(int dom)
{
    static const s_name_t	names[] =
    {
	S_NM(LX_AF_UNSPEC),
	S_NM(LX_AF_UNIX),
	S_NM(LX_AF_INET),
	S_NM(LX_AF_AX25),
	S_NM(LX_AF_IPX),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(dom, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", dom);
    return buf;
}

static const char *
ntv_sockdomstr(int dom)
{
    static const s_name_t	names[] =
    {
	S_NM(AF_UNSPEC),
	S_NM(AF_UNIX),
	S_NM(AF_INET),
	S_NM(AF_IMPLINK),
	S_NM(AF_PUP),
	S_NM(AF_CHAOS),
	S_NM(AF_NS),
#ifdef AF_ISO
	S_NM(AF_ISO),
#endif
	S_NM(AF_ECMA),
	S_NM(AF_DATAKIT),
	S_NM(AF_CCITT),
	S_NM(AF_SNA),
	S_NM(AF_DECnet),
	S_NM(AF_DLI),
	S_NM(AF_LAT),
	S_NM(AF_HYLINK),
	S_NM(AF_APPLETALK),
	S_NM(AF_ROUTE),
	S_NM(AF_LINK),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(dom, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", dom);
    return buf;
}

static const char *
lx_socktypestr(int type)
{
    static const s_name_t	names[] =
    {
	S_NM(LX_SOCK_STREAM),
	S_NM(LX_SOCK_DGRAM),
	S_NM(LX_SOCK_RAW),
	S_NM(LX_SOCK_RDM),
	S_NM(LX_SOCK_SEQPACKET),
	S_NM(LX_SOCK_PACKET),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(type, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", type);
    return buf;
}

static const char *
ntv_socktypestr(int type)
{
    static const s_name_t	names[] =
    {
	S_NM(SOCK_STREAM),
	S_NM(SOCK_DGRAM),
	S_NM(SOCK_RAW),
	S_NM(SOCK_RDM),
	S_NM(SOCK_SEQPACKET),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(type, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", type);
    return buf;
}

static const char *
lx_sockprotostr(int proto)
{
    static const s_name_t	names[] =
    {
	S_NM(LX_PF_UNSPEC),
	S_NM(LX_PF_UNIX),
	S_NM(LX_PF_INET),
	S_NM(LX_PF_AX25),
	S_NM(LX_PF_IPX),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(proto, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", proto);
    return buf;
}

static const char *
ntv_sockprotostr(int proto)
{
    static const s_name_t	names[] =
    {
	S_NM(PF_UNSPEC),
	S_NM(PF_UNIX),
	S_NM(PF_INET),
	S_NM(PF_IMPLINK),
	S_NM(PF_PUP),
	S_NM(PF_CHAOS),
	S_NM(PF_NS),
	S_NM(PF_ECMA),
	S_NM(PF_DATAKIT),
	S_NM(PF_CCITT),
	S_NM(PF_SNA),
	S_NM(PF_DECnet),
	S_NM(PF_DLI),
	S_NM(PF_LAT),
	S_NM(PF_HYLINK),
	S_NM(PF_APPLETALK),
	S_NM(PF_ROUTE),
	S_NM(PF_LINK),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(proto, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", proto);
    return buf;
}

static const char *
lx_sockoptstr(int opt)
{
    static const s_name_t	names[] =
    {
	S_NM(LX_SO_DEBUG),
	S_NM(LX_SO_REUSEADDR),
	S_NM(LX_SO_TYPE),
	S_NM(LX_SO_ERROR),
	S_NM(LX_SO_DONTROUTE),
	S_NM(LX_SO_BROADCAST),
	S_NM(LX_SO_SNDBUF),
	S_NM(LX_SO_RCVBUF),
	S_NM(LX_SO_KEEPALIVE),
	S_NM(LX_SO_OOBINLINE),
	S_NM(LX_SO_NO_CHECK),
	S_NM(LX_SO_PRIORITY),
	S_NM(LX_SO_LINGER),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(opt, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", opt);
    return buf;
}

static const char *
ntv_sockoptstr(int opt)
{
    static const s_name_t	names[] =
    {
	S_NM(SO_DEBUG),
	S_NM(SO_ACCEPTCONN),
	S_NM(SO_REUSEADDR),
	S_NM(SO_KEEPALIVE),
	S_NM(SO_DONTROUTE),
	S_NM(SO_BROADCAST),
	S_NM(SO_USELOOPBACK),
	S_NM(SO_LINGER),
	S_NM(SO_OOBINLINE),
#ifdef SO_ORDREL
	S_NM(SO_ORDREL),
#endif
#ifdef SO_IMASOCKET
	S_NM(SO_IMASOCKET),
#endif
#ifdef SO_MGMT
	S_NM(SO_MGMT),
#endif
#ifdef SO_REUSEPORT
	S_NM(SO_REUSEPORT),
#endif
#ifdef SO_LISTENING
	S_NM(SO_LISTENING),
#endif
	S_NM(SO_SNDBUF),
	S_NM(SO_RCVBUF),
	S_NM(SO_SNDLOWAT),
	S_NM(SO_RCVLOWAT),
	S_NM(SO_SNDTIMEO),
	S_NM(SO_RCVTIMEO),
	S_NM(SO_ERROR),
	S_NM(SO_TYPE),
	S_NM(SO_PROTOTYPE),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(opt, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", opt);
    return buf;
}

static const char *
lx_socklvlstr(int level)
{
    static const s_name_t	names[] =
    {
	S_NM(LX_SOL_SOCKET),
	S_NM(IPPROTO_IP),
	S_NM(IPPROTO_ICMP),
	S_NM(IPPROTO_IGMP),
#ifdef IPPROTO_IPIP
	S_NM(IPPROTO_IPIP),
#endif
	S_NM(IPPROTO_TCP),
	S_NM(IPPROTO_EGP),
	S_NM(IPPROTO_PUP),
	S_NM(IPPROTO_UDP),
	S_NM(IPPROTO_IDP),
	S_NM(IPPROTO_RAW),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(level, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", level);
    return buf;
}

static const char *
ntv_socklvlstr(int level)
{
    static const s_name_t	names[] =
    {
	S_NM(SOL_SOCKET),
	S_NM(IPPROTO_IP),
	S_NM(IPPROTO_ICMP),
	S_NM(IPPROTO_IGMP),
	S_NM(IPPROTO_GGP),
	S_NM(IPPROTO_TCP),
	S_NM(IPPROTO_EGP),
	S_NM(IPPROTO_PUP),
	S_NM(IPPROTO_UDP),
	S_NM(IPPROTO_IDP),
#ifdef IPPROTO_TP
	S_NM(IPPROTO_TP),
#endif
#ifdef IPPROTO_HELLO
	S_NM(IPPROTO_HELLO),
#endif
#ifdef IPPROTO_ND
	S_NM(IPPROTO_ND),
#endif
#ifdef IPPROTO_EON
	S_NM(IPPROTO_EON),
#endif
#ifdef IPPROTO_RAW
	S_NM(IPPROTO_RAW),
#endif
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(level, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", level);
    return buf;
}

static const char *
lx_sockcallstr(int call)
{
    static const s_name_t	names[] =
    {
	S_NM(LX_SYS_SOCKET),
	S_NM(LX_SYS_BIND),
	S_NM(LX_SYS_CONNECT),
	S_NM(LX_SYS_LISTEN),
	S_NM(LX_SYS_ACCEPT),
	S_NM(LX_SYS_GETSOCKNAME),
	S_NM(LX_SYS_GETPEERNAME),
	S_NM(LX_SYS_SOCKETPAIR),
	S_NM(LX_SYS_SEND),
	S_NM(LX_SYS_RECV),
	S_NM(LX_SYS_SENDTO),
	S_NM(LX_SYS_RECVFROM),
	S_NM(LX_SYS_SHUTDOWN),
	S_NM(LX_SYS_SETSOCKOPT),
	S_NM(LX_SYS_GETSOCKOPT),
	S_NM_END
    };
    static char buf[32];
    const char	*tp;

    if ((tp = strname(call, names)) != NULL)
	return tp;

    sprintf(buf, "<Unknown %d>", call);
    return buf;
}

static const char *
strname(int value, const s_name_t *np)
{
    if (np)
	for ( ; np->name; ++np)
	    if (np->value == value)
		return np->name;

    if (value == -1)
	return "UNSUPPORTED";

    return NULL;
}

#define FDSET_CNT	3

static const char *
fdsetstr(int nfd, fd_set *fds)
{
    static char fileNoBuf[FDSET_CNT][FD_SETSIZE + 1];
    static int	bufIx = FDSET_CNT;
    char	*buf;
    char	*tp;
    int 	fd;

    if (fds == NULL)
	return ("<NULL>");
    if (++bufIx >= FDSET_CNT)
	bufIx = 0;

    tp = buf = &fileNoBuf[bufIx][0];

    *tp++ = '[';
    for (fd = 0; fd < nfd; ++fd)
    {
	if (!FD_ISSET(fd, fds))
	    continue;

	sprintf(tp, "%d", fd);
	tp += strlen(tp);
	if (fd+1 < nfd)
	{
	    if (FD_ISSET(fd+1, fds))
	    {
		*tp++ = '-';
		while ((fd+1 < nfd) && FD_ISSET(fd+1, fds))
		    ++fd;
		sprintf(tp, "%d", fd);
		tp += strlen(tp);

		if (fd+1 < nfd)
		    *tp++ = ',';
	    }
	    else
		*tp++ = ',';
	}
    }

    *tp++ = ']';
    *tp = '\0';
    return buf;
}
