#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 *
 * This file replaces libsocket.a functions so we can do a
 * static link on UnixWare 7.
 */

#include	<errno.h>
#include	<sys/time.h>
#include	<sys/systeminfo.h>
#include	<unistd.h>

#ifdef UW7
#   include	<sys/syscall.h>
#   define	_INKERNEL
#   include	<sys/socket.h>
#   undef SOCKETSYS
#   define SOCKETSYS	140
#   define SO_SOCKPAIR	27
#else
#   include	<sys/socket.h>
#endif

#include	"lxrun.h"
#include	"trace.h"

int
socket(int family, int type, int protocol)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: socket(%d, %d, %d)\n", family, type, protocol));

    switch (type)
    {
	case SOCK_DGRAM:
	    type = 2;
	    break;

	case SOCK_STREAM:
	case SOCK_RAW:
	case SOCK_RDM:
	case SOCK_SEQPACKET:
	    type--;
	    break;
    }

    sreq.args[0] = SO_SOCKET;
    sreq.args[1] = family;
    sreq.args[2] = type;
    sreq.args[3] = protocol;
    return syscall(SOCKETSYS, &sreq);
}

int
socketpair(int family, int type, int protocol, int *sock)
{
    struct socksysreq	sreq;
    int			ret;
    int			sock1;
    int			sock2;

    SOCK_TRACE_MSG(("TRACE: socketpair(%d, %d, %d, 0x%x)\n",
					family, type, protocol, sock));

    if (family != AF_UNIX)
    {
	SOCK_TRACE_MSG(("TRACE: socketpair() bad family %d\n", family));
	return enoany( EPROTONOSUPPORT, "socketpair bad family");
    }

    if ((sock1 = socket(family, type, protocol)) == -1)
    {
	SOCK_TRACE_MSG(("TRACE: socketpair() sock1 fail %d\n", errno));
	return -1;
    }

    if ((sock2 = socket(family, type, protocol)) == -1)
    {
	SOCK_TRACE_MSG(("TRACE: socketpair() sock2 fail %d\n", errno));
	close(sock1);
	return -1;
    }

    sreq.args[0] = SO_SOCKPAIR;
    sreq.args[1] = (int)sock1;
    sreq.args[2] = (int)sock2;

    if ((ret = syscall(SOCKETSYS, &sreq)) == -1)
    {
	close(sock1);
	close(sock2);
	return -1;
    }

    sock[0] = sock1;
    sock[1] = sock2;
    return 0;
}

int
recv(int sock, void *buf, size_t len, int flags)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: recv(%d, 0x%x, %d, %d)\n", sock, buf, len, flags));

    sreq.args[0] = SO_RECV;
    sreq.args[1] = sock;
    sreq.args[2] = (int)buf;
    sreq.args[3] = len;
    sreq.args[4] = flags;
    return syscall(SOCKETSYS, &sreq);
}

int
send(int sock, const void *msg, size_t len, int flags)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: send(%d, 0x%x, %d, %d)\n", sock, msg, len, flags));

    sreq.args[0] = SO_SEND;
    sreq.args[1] = sock;
    sreq.args[2] = (int)msg;
    sreq.args[3] = len;
    sreq.args[4] = flags;
    return syscall(SOCKETSYS, &sreq);
}

int
listen(int sock, int backlog)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: listen(%d, %d)\n", sock, backlog));

    sreq.args[0] = SO_LISTEN;
    sreq.args[1] = sock;
    sreq.args[2] = backlog;
    return syscall(SOCKETSYS, &sreq);
}

int
getsockopt(int sock, int level, int optname, void *optval, size_t *optlen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: getsockopt(%d, %d, %d, 0x%x, 0x%x)\n",
				    sock, level, optname, optval, optlen));

    sreq.args[0] = SO_GETSOCKOPT;
    sreq.args[1] = sock;
    sreq.args[2] = level;
    sreq.args[3] = optname;
    sreq.args[4] = (int)optval;
    sreq.args[5] = (int)optlen;
    return syscall(SOCKETSYS, &sreq);
}

int
setsockopt(int sock, int level, int optname, const void *optval, size_t optlen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: setsockopt(%d, %d, %d, 0x%x, %d)\n",
				    sock, level, optname, optval, optlen));

    sreq.args[0] = SO_SETSOCKOPT;
    sreq.args[1] = sock;
    sreq.args[2] = level;
    sreq.args[3] = optname;
    sreq.args[4] = (int)optval;
    sreq.args[5] = optlen;
    return syscall(SOCKETSYS, &sreq);
}

int
shutdown(int sock, int how)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: shutdown(%d, %d)\n", sock, how));

    sreq.args[0] = SO_SHUTDOWN;
    sreq.args[1] = sock;
    sreq.args[2] = how;
    return syscall(SOCKETSYS, &sreq);
}

int
setdomainname(char *name, int namelen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: setdomainname(%s, %d)\n", name, namelen));

    return sysinfo(SI_SET_SRPC_DOMAIN, name, namelen + 1);
}

int
getdomainname(char *name, int namelen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: getdomainname(%#x, %d)\n", name, namelen));

    return sysinfo(SI_SRPC_DOMAIN, name, namelen);
}

int
accept(int sock, struct sockaddr *addr, size_t *addrlen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: accept(%d, 0x%x, 0x%x)\n", sock, addr, addrlen));

    sreq.args[0] = SO_ACCEPT;
    sreq.args[1] = sock;
    sreq.args[2] = (int)addr;
    sreq.args[3] = (int)addrlen;
    return syscall(SOCKETSYS, &sreq);
}

int
connect(int sock, const struct sockaddr *addr, size_t addrlen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: connect(%d, 0x%x, 0x%x)\n", sock, addr, addrlen));

    sreq.args[0] = SO_CONNECT;
    sreq.args[1] = sock;
    sreq.args[2] = (int)addr;
    sreq.args[3] = addrlen;
    return syscall(SOCKETSYS, &sreq);
}

int
getsockname(int sock, struct sockaddr *name, size_t *namelen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: getsockname(%d, 0x%x, 0x%x)\n", sock, name, namelen));

    sreq.args[0] = SO_GETSOCKNAME;
    sreq.args[1] = sock;
    sreq.args[2] = (int)name;
    sreq.args[3] = (int)namelen;
    return syscall(SOCKETSYS, &sreq);
}

int
bind(int sock, const struct sockaddr *name, size_t namelen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: bind(%d, 0x%x, %d)\n", sock, name, namelen));

    sreq.args[0] = SO_BIND;
    sreq.args[1] = sock;
    sreq.args[2] = (int)name;
    sreq.args[3] = namelen;
    return syscall(SOCKETSYS, &sreq);
}

int
recvfrom(int sock, void *buf, size_t len, int flags,
				struct sockaddr *from, size_t *fromlen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: recvfrom(%d, 0x%x, %d, %d, 0x%x, 0x%x)\n",
				sock, buf, len, flags, from, fromlen));

    sreq.args[0] = SO_RECVFROM;
    sreq.args[1] = sock;
    sreq.args[2] = (int)buf;
    sreq.args[3] = len;
    sreq.args[4] = flags;
    sreq.args[5] = (int)from;
    sreq.args[6] = (int)fromlen;
    return syscall(SOCKETSYS, &sreq);
}

int
getpeername(int sock, struct sockaddr *name, size_t *namelen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: getpeername(%d, 0x%x, 0x%x)\n", sock, name, namelen));

    sreq.args[0] = SO_GETPEERNAME;
    sreq.args[1] = sock;
    sreq.args[2] = (int)name;
    sreq.args[3] = (int)namelen;
    return syscall(SOCKETSYS, &sreq);
}

int
sendto(int sock, const void *msg, size_t len, int flags,
				    const struct sockaddr *to, size_t tolen)
{
    struct socksysreq	sreq;

    SOCK_TRACE_MSG(("TRACE: sendto(%d, 0x%x, %d, %d, 0x%x, %d)\n",
					sock, msg, len, flags, to, tolen));

    sreq.args[0] = SO_SENDTO;
    sreq.args[1] = sock;
    sreq.args[2] = (int)msg;
    sreq.args[3] = len;
    sreq.args[4] = flags;
    sreq.args[5] = (int)to;
    sreq.args[6] = tolen;
    return syscall(SOCKETSYS, &sreq);
}


#ifdef ORIG_CODE
int socketpair()	{ return -1; }
int recv()		{ return -1; }
int socket()		{ return -1; }
int send()		{ return -1; }
int listen()		{ return -1; }
int setsockopt()	{ return -1; }
int shutdown()		{ return -1; }
int setdomainname()	{ return -1; }
int getdomainname()	{ return -1; }
#endif
