#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z%%M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<assert.h>
#include	<errno.h>
#include	<signal.h>
#include	<stdlib.h>
#include	<string.h>
#include	<sys/siginfo.h>
#include	<sys/regset.h>
#include	<sys/reg.h>
#include	<ucontext.h>
#include	"lxrun.h"
#include	"sysent.h"
#include	"trace.h"

typedef union
{
	void	*p;
	int	i;
} reg_t;

#define	ARG0	regs[EBX].i
#define	ARG1	regs[ECX].i
#define	ARG2	regs[EDX].i
#define	ARG3	regs[ESI].i
#define	ARG4	regs[EDI].i

extern void deliver_signal(int sig, siginfo_t *info, void *context);

/*
 * the following four declarations are actually all labels
 * in the text segment but they are declared here as extern
 * arrays of unsigned char to emphasise the fact that they are
 * not callable functions
 */
extern uchar_t getbyte_can_fault_here[];
extern uchar_t putbyte_can_fault_here[];
extern uchar_t getbyte_fault[];
extern uchar_t putbyte_fault[];

void
syscall_handler(
	int		sig,
	siginfo_t	*info,
	void		*context
	)
{
	reg_t		*regs;
	uchar_t		*eip;

	assert(sig == SIGSEGV && context != NULL);

#if defined(OSR5)
	regs	= (reg_t *)((ucontext_t *)context)->uc_mcontext.regs;
#else
	regs	= (reg_t *)((ucontext_t *)context)->uc_mcontext.gregs;
#endif

	eip	= regs[EIP].p;

#if 0
        /*  Low level and normally not very useful */
         SYS_TRACE_MSG(("SYSCALL/SIGSEGV: eip 0x%x, peek 0x%2x 0x%2x,"
                       "faulted on 0x%x\n",
			 eip, getbyte(eip), getbyte(eip+1),
			 info->si_addr));
#endif			 
			 
	if (eip == getbyte_can_fault_here)
	{
		regs[EIP].p = getbyte_fault;
	}
	else if (eip == putbyte_can_fault_here)
	{
		regs[EIP].p = putbyte_fault;
	}
	else if (getbyte(eip) == 0xcd && getbyte(eip+1) == 0x80)
	{
		int	n;
		int	rval = -1;

		regs[EIP].i	+= 2;
		n		= regs[EAX].i;	/* system call #	*/
		errno		= 0;

	        SYS_TRACE_CALL( trace_syscall_entry(n, &regs[0].i));

	    
		if (n < 0 || n >= nsysent)
		{
		    errno = ENOSYS;
		}
		else
		{
			char path0[PATH_MAX], path1[PATH_MAX];
			char *arg0 = (char *)ARG0;
			char *arg1 = (char *)ARG1;

			/*
			 * Only arguments 0 and 1 might be remappable paths.
			 */
			if (is_remappable(n, 0))
			{
			    if ((rval = lx_pathname(arg0, path0,
						    sizeof (path0))) == -1)
				goto out;
			    if((do_trace & TRACE_SYS) &&
						((do_trace & TRACE_VERBOSE) ||
						strcmp(arg0, path0) != 0))
				SYS_TRACE_MSG(("PATHMAP: '%s' --> '%s'%c",
					arg0,path0,(do_trace & TRACE_VERBOSE)?
							'\n' : ' '));
			    arg0 = path0;
			}
			if (is_remappable(n, 1))
			{
			    if ((rval = lx_pathname(arg1, path1,
						    sizeof (path1))) == -1)
				goto out;
			    if((do_trace & TRACE_SYS) &&
						((do_trace & TRACE_VERBOSE) ||
						strcmp(arg0, path0) != 0))
				SYS_TRACE_MSG(("PATHMAP: '%s' --> '%s'%c",
					arg1,path1,(do_trace & TRACE_VERBOSE)?
							'\n' : ' '));
			    arg1 = path1;
			}
			rval = sysent[n].syscall(arg0, arg1, ARG2, ARG3, ARG4);
		}
		
	out:
		regs[EAX].i = (rval == -1) ? - lx_errno(errno) : rval;

		/* If we just forked, set up a new trace file */
		TRACE_CALL(TRACE_ALL, if (n == 2) trace_fork());

	        SYS_TRACE_CALL( trace_syscall_result(n, rval, errno));


		/*
		 * When we return from the signal handler, setcontext()
		 * will be called.  That restores a number of things,
		 * including the signal mask.  So if we made the Linux
		 * sigprocmask() system call, we should stop setcontext()
		 * from blowing away our freshly modified signal mask.
		 *
		 * XXX: This is ugly as sin.
		 */
		if (n == 126)	/* __NR_sigprocmask */
			((ucontext_t *)context)->uc_flags &= ~UC_SIGMASK;

#if defined(PATHMAPFILE) && defined(PHANTOM_MAP_FILE)
		/* Clean up any phantom files that had to be created. */
		remove_phantom_file();
#endif
	}
	else
	{
            SYS_TRACE_CALL(trace_unexpected_sigsegv((void **)regs, eip));
	    deliver_signal(sig, info, context);
	}
}
