#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z%%M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */


#include	<errno.h>
#include	<stdarg.h>
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<signal.h>
#include	<fcntl.h>
#include	<sys/time.h>
#include	<sys/reg.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<sys/resource.h>

#include	"lxrun.h"
#include	"sysent.h"
#include	"linux/ipc.h"
#include	"linux/socket.h"
#include	"linux/sigcontext.h"
#include	"trace.h"

#define	ARG0	regs[EBX]
#define	ARG1	regs[ECX]
#define	ARG2	regs[EDX]
#define	ARG3	regs[ESI]
#define	ARG4	regs[EDI]

#define	SYS(scall, rtype, args)	{ #scall, 0, rtype, { args } },

unsigned long	do_trace = 0;

struct trace_info syscall_info[] =
{
#include "syscalls.h"
};



/* log use of missing syscall, setup ENOSYS errno and return failure value  */
int
enosys(const char *msg)
{
	warning(0, "*** UNIMPLEMENTED syscall %s called ***", msg);

	TRACE_MSG( TRACE_ALL, ("*** UNIMPLEMENTED syscall %s called ***\n",
			       msg));
				    
	errno = ENOSYS;
	return -1;
}

/* Trace use of non emulated syscall mode or other failure
 * - unsupported submodes to syscall shld return EINVAL but existing
 * use is combination of EINVAL and ENOSYS
 * Setup arbitrary errno and return failure value
 */
int
enoany(int r_errno, const char *msg)
{
    if (msg)
	TRACE_MSG( TRACE_ALL, ("%s\n", msg));

    errno = r_errno;
    return -1;
}


/*
 * If argnum is 'path' in syscall_info its a pathname and  remappable
 */
int
is_remappable(int sysnum, int argnum)
{
	return (syscall_info[sysnum].argtypes[argnum] == a_path);
}


struct trace_info socket_info[] =
{
	SYS(socketcall,		RET(int), ARGS1(int))
	SYS(socket,		RET(int), ARGS3(int, int, int))
	SYS(bind,		RET(int), ARGS3(int, ptr, int))
	SYS(connect,		RET(int), ARGS3(int, ptr, int))
	SYS(listen,		RET(int), ARGS2(int, int))
	SYS(accept,		RET(int), ARGS3(int, ptr, ptr))
	SYS(getsockname,	RET(int), ARGS3(int, ptr, ptr))
	SYS(getpeername,	RET(int), ARGS3(int, ptr, ptr))
	SYS(socketpair,		RET(int), ARGS4(int, int, int, ptr))
	SYS(send,		RET(int), ARGS4(int, ptr, int, int))
	SYS(sendto,		RET(int), ARGS6(int, ptr, int, int, ptr, int))
	SYS(recv,		RET(int), ARGS4(int, ptr, int, int))
	SYS(recvfrom,		RET(int), ARGS6(int, ptr, int, int, ptr, ptr))
	SYS(shutdown,		RET(int), ARGS2(int, int))
	SYS(setsockopt,		RET(int), ARGS5(int, int, int, ptr, int))
	SYS(getsockopt,		RET(int), ARGS5(int, int, int, ptr, ptr))
};

struct trace_info ipc_info[] =
{
	SYS(ipc,		RET(int), ARGS1(int))
	SYS(semop,		RET(int), ARGS3(int, ptr, int))
	SYS(semget,		RET(int), ARGS3(int, int, ptr))
	SYS(semctl,		RET(int), ARGS4(int, int, hex, ptr))
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(msgsnd,		RET(int), ARGS4(int, ptr, int, ptr))
	SYS(msgrcv,		RET(int), ARGS5(int, ptr, int, ptr, ptr))
	SYS(msgget,		RET(int), ARGS2(int, int))
	SYS(msgctl,		RET(int), ARGS3(int, int, ptr))
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(ipc,		RET(int), ARGS0())
	SYS(shmat,		RET(int), ARGS4(int, ptr, int, hex))
	SYS(shmdt,		RET(int), ARGS1(ptr))
	SYS(shmget,		RET(int), ARGS3(int, int, hex))
	SYS(shmctl,		RET(int), ARGS3(int, int, hex))
};

static FILE	*trace_fp;
static int	pid = -1;
static int	masterpid = -1;

typedef struct
{
    const char		*key;
    unsigned long	bits;
} trace_opts_t;

static const trace_opts_t	trace_opts[] =
{
    { "all",		TRACE_ALL },
    { "sys",		TRACE_SYS },	/* All system call arguments */
    { "sig",		TRACE_SIG },	/* All signal tracing */
    { "sock",		TRACE_SOCK },
    { "select",		TRACE_SELECT },
    { "read",		TRACE_READ },
    { "write",		TRACE_WRITE },
    { "open",		TRACE_OPEN },
    { "access",		TRACE_ACCESS },
    { "fcntl",		TRACE_FCNTL },
    { "ioctl",		TRACE_IOCTL },
    { "brk",		TRACE_BRK },
    { "exec",		TRACE_EXEC },
#ifdef OLD_PROCFS
    { "procfs",		TRACE_PROCFS },
#endif
    { "pathmap",	TRACE_PATHMAP },
    { "mmap",		TRACE_MMAP },
    { "elf",		TRACE_ELF },
    { "verbose",	TRACE_VERBOSE|TRACE_SYS },
    { NULL }				/* End of table */
};

void
trace_help()
{
    const trace_opts_t	*op;
    int			col;

    fprintf(stderr, "\nTrace elements are:\n\t");
    col = 0;
    for (op = trace_opts; op->key; ++op)
    {
	if (col >= 60)
	{
	    fputs("\n\t", stdout);
	    col = 0;
	}
	else if (op != trace_opts)
	{
	    putc(' ', stderr);
	    ++col;
	}

	fputs(op->key, stderr);
	col += strlen(op->key);
    }

    putc('\n', stderr);
}

/*
 * This routine is called for every -t command line option.  The
 * value of do_trace will be accumulated.
 */

int
trace_init(const char *element)
{
    const trace_opts_t	*op;
    static char		buf[256];
    char		*envstr;
    char		*key;
    unsigned long       entry_dt;
    
    if (!element)
    	return 0;	/* Nothing to set */

    if (do_trace)	/* already set and reinitting */	
	return 0;
    
    entry_dt = do_trace;
    strncpy(buf, element, sizeof(buf));

    /* Make sure children inherit trace options */
    envstr = (char *)malloc(strlen(element)+13);
    strcpy(envstr, "LXRUN_TRACE=");
    strcat(envstr, element);
    putenv(envstr);

    if ((key = strtok(buf, ",")) != NULL)
	do
	{
	    for (op = trace_opts; op->key; ++op)
		if (strcmp(key, op->key) == 0)
		{
		    do_trace |= op->bits;
		    break;
		}

	    if (!op->key)
	    {
		fprintf(stderr, "Unknown trace element: %s\n", key);
		return EINVAL;
	    }
	}
	while ((key = strtok(NULL, ",")) != NULL);

        if ( do_trace && !entry_dt)
            fprintf(stderr, "lxrun: Tracing to %s/lxrun.%lu (mode 0x%8.8lx)\n",
			TRACE_DIR, getpid(), do_trace);
                                        
    return 0;
}

/*
 * TRACE printf()
 */

void
tprintf(char *fmt, ...)
{
	va_list	args;
	struct rlimit rlim;
	int	trace_fd;

	if (trace_fp == NULL)
	{
		char trace_name[PATH_MAX];
		struct stat st;

	        if (masterpid == -1)
	            sprintf(trace_name, "%s/lxrun.%lu", TRACE_DIR, getpid());
		else
	            sprintf(trace_name, "%s/lxrun.%lu-%lu", TRACE_DIR,
	                        masterpid, getpid());

		/*
	         * For security, we check to make sure the file
        	 * doesn't already exist.  It's possible some knave
	         * symlinked /etc/passwd to /tmp/lxrun.123 or something.
        	 *
	         * Unfortunately, that means that if we simply exec, we can
        	 * no longer log.
		 */
#if 0
    	        if ((lstat(trace_name, &st) != -1) || (errno != ENOENT))
		{
		    warning(0,
			    "lxrun: Warning: File %s already exists.\n"
			    "        -- no trace output will be produced.\n",
			    trace_name);
		    trace_fd = open("/dev/null", O_WRONLY);
		}
		else
#endif
		trace_fd = open(trace_name,
			    O_WRONLY | O_TRUNC | O_CREAT, 0666);
		if(trace_fd < 0)
			fatal(errno, "failed to open trace file");
		/*
		 * Now be sneaky and try to dup the fd above the soft
		 * limit, but below the hard limit. This is to avoid breaking
		 * when programs go close all their open file descriptors,
		 * which would confuse stdio.
		 */
		if ((getrlimit(RLIMIT_NOFILE, &rlim) != -1) &&
		    (rlim.rlim_cur < rlim.rlim_max))
		{
			rlim_t	oldsoft;
			int	newfd, tmpfd;

			oldsoft = rlim.rlim_cur;
			rlim.rlim_cur = rlim.rlim_max;
			if (setrlimit(RLIMIT_NOFILE, &rlim) != -1)
			{
			    for (newfd = oldsoft + 1;
				 newfd < rlim.rlim_max; newfd++)
			    {
				/* see if 'newfd' is already taken */
				tmpfd = dup(newfd);
				if (tmpfd != -1)
				{
				    (void) close(tmpfd);
				    continue;
				}
				else if (errno != EBADF)
				{
				    continue;
				}
				if (dup2(trace_fd, newfd) != -1)
				{
				    (void) close(trace_fd);
				    trace_fd = newfd;
				    break;
				}
			    }
			    /* restore old soft limit */
			    rlim.rlim_cur = oldsoft;
			    (void) setrlimit(RLIMIT_NOFILE, &rlim);
			}
		}
		if(fcntl(trace_fd, F_SETFD, FD_CLOEXEC) < 0)
			fatal(errno, "set close-on-exec of trace file failed");
		if((trace_fp = fdopen(trace_fd, "w"))==NULL)
			fatal(errno, "failed to fdopen trace_fd");

		setbuf(trace_fp, NULL);
	}

	va_start(args, fmt);
	(void) vfprintf(trace_fp, fmt, args);
	va_end(args);
}

void
trace_syscall_result(int n, int rval, int err)
{
	char			buf[256];
	struct trace_info	*t;

	if (n >= 0 && n < nsysent)
		t = &syscall_info[n];
	else
	{
		tprintf(" unknown syscall (%d)\n", n);
		return;
	}

	if (rval == -1)
	{
	    sprintf(buf, "FAILED: %s (%d)", strerror(err), err);
	}
	else
	{
	    strcpy(buf, "returned ");
	    PRINT_RET(t, buf, rval);
	}
	if(do_trace & TRACE_VERBOSE)
	    tprintf("SYSCALL: %s %s\n", t->name, buf);
	else
	    tprintf("%s\n", buf);
}

void
trace_syscall_entry(int n, int *regs)
{
	struct	sysent	*s;
	int	args[8];
	char	buf[256];
	char	*p;

	args[0]	= ARG0;
	args[1]	= ARG1;
	args[2]	= ARG2;
	args[3]	= ARG3;
	args[4]	= ARG4;
	args[5]	= 0;
	args[6]	= 0;
	args[7]	= 0;

	if (pid == -1)
		pid = getpid();

	if (n >= 0 && n < nsysent)
	{
		int	i;
		struct trace_info	*t;

		t = &syscall_info[n];
		sprintf(buf, "%s(", t->name);
		p = &buf[strlen(buf)];

		for (i = 0; i < 6 && t->argtypes[i] != 0;)
		{
			if (i > 0)
				strcat(buf, ", ");
			i += PRINT_ARG(t, buf, args, i);
		}

		strcat(buf, ")");

	    if(do_trace & TRACE_VERBOSE)
		tprintf("\nSYSCALL: %s\n", buf);
	    else
		tprintf("%s ", buf);
	}
	else
	    {
	    if(do_trace & TRACE_VERBOSE)
		tprintf("\nSYSCALL: UNKNOWN syscall %d\n", n);
	    else
		tprintf("UNKNOWN syscall %d\n", n);
	    }
}


void
trace_fork(void)
{
	/*
	** This should get called when we fork().
	** This is where we arrange for a new trace file to be opened. 
	*/

	int newpid = getpid();
	if (newpid != pid)	/* If we're a child... */
		{
		/* Keep the original lxrun invocation's pid */
		if(masterpid == -1)
			masterpid = getppid();

		/* Start a new trace file */
		trace_fp = NULL;

		pid = newpid;
		}
}


void
trace_signal(int sig, void *info, void *context)
{
    tprintf("TRACE: pid %d: signal(%d = %s, %#X, %#X)\n",
				pid, sig, os_sigName(sig), info, context);
}

struct stackframe
{
    struct stackframe	*fp;		/* next stack frame	*/
    unsigned char	*ret;		/* return address	*/
    unsigned		arg[1];		/* function arguments	*/
};


static void
prname(unsigned long addr)
{
    if (addr == 0)
	tprintf("????????");
    else
	tprintf("%8x", addr);
}

/*
 * prframe()	- try to print out the contents of a stackframe
 *
 *		   understands 4 types of call:
 *			   e8 xx xx xx xx	call	xx xx xx xx
 *			ff 15 xx xx xx xx	call	ds:[xx xx xx xx]
 *			         ff 55 xx	call	ss:[ebp + xx]
 *			ff 95 xx xx xx xx	call	ss:[ebp + xx xx xx xx]
 */
static void
prframe(struct stackframe *fp)
{
    unsigned char	*r;
    unsigned long	f;
    int			ac;
    int			i;

    r = fp->ret;

    if (r[-5] == 0xe8)
	f = (unsigned long)r + *(long *)(r - 4);
    else if (r[-6] == 0xff && r[-5] == 0x15)
	f = **(unsigned long **)(r - 4);
    else if (r[-6] == 0xff && r[-5] == 0x95)
	f = *(unsigned long *)((unsigned char *)fp->fp + *(long *)(r - 4));
    else if (r[-3] == 0xff && r[-2] == 0x55)
	f = *(unsigned long *)((unsigned char *)fp->fp + *(char *)(r - 1));
    else
	f = 0;
    
    prname(f);		/* print function name	*/
    tprintf("(");

    if (r[0] == 0x83 && r[1] == 0xc4)		/* add esp, imm8	*/
	ac = r[2] / 4;
    else if (r[0] == 0x81 && r[1] == 0xc4)	/* add esp, imm32	*/
	ac = *(unsigned long *)&r[2] / 4;
    else if (r[0] == 0x59)			/* pop ecx		*/
	for (ac = 0; r[ac] == 0x59; ac++)
	    ;
    else
	ac = 0;

    for (i = 0; i < ac; i++)
	tprintf("%x%s", fp->arg[i], (i == ac - 1) ? "" : ", ");

    tprintf(")\t called from ");
    prname((unsigned long)r);
    tprintf("\n");
}


static void
trace_stack(void *frame)
{
    struct stackframe	*fp;

    fp	= frame;
    for (fp = fp->fp; fp->fp != NULL; fp = fp->fp)
	prframe(fp);

    tprintf("\n");
}


void
trace_reg_changes(unsigned long *regs, struct lx_sigcontext_struct *lx_context)
{
	if (do_trace == 0)
		return;

#define CHECK(REG, reg) \
	if (regs[REG] != lx_context->reg) \
		tprintf(#REG " changed from %8x to %8x\n", \
		    regs[REG], lx_context->reg)

	CHECK(EDI, edi);
	CHECK(ESI, esi);
	CHECK(EBP, ebp);
	CHECK(ESP, esp);
	CHECK(EBX, ebx);
	CHECK(EDX, edx);
	CHECK(ECX, ecx);
	CHECK(EAX, eax);
	CHECK(EIP, eip);
	CHECK(EFL, eflags);
#undef CHECK
}

void
trace_unexpected_sigsegv(void **regs, uchar_t *eip)
{
	unsigned long *sp = regs[UESP];

	if (do_trace == 0)
		return;

	tprintf("\nCaught signal 11 (SIGSEGV) at EIP = %#x\n", eip);
	tprintf("ESP = %08x, EBP = %08x, ESI = %08x, EDI = %08x\n",
	    regs[UESP], regs[EBP], regs[ESI], regs[EDI]);
	tprintf("EAX = %08x, EBX = %08x, ECX = %08x, EDX = %08x\n",
	    regs[EAX], regs[EBX], regs[ECX], regs[EDX]);

	tprintf("Stack top: %08x %08x %08x %08x\n",
	    sp[-4], sp[-3], sp[-2], sp[-1]);

	tprintf("Raising SIGSTOP on pid %d for a debugger.\n",
	    getpid());
	kill(getpid(), SIGSTOP);
	trace_stack(regs[EBP]);
	exit(1);
}
