#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif
#ifdef SOLARIS
#pragma ident "%Z% %M% %I% %D% SMI"
#endif

/*
 * Copyright (C) 1997 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#if !defined(TRACE_H)
#define TRACE_H

#undef TRACE_ARG
#define	TRACE_ARG(name)	a_##name,
enum arg_type
{
#include "traceargs.h"
    a_num_arg_types
};
#undef TRACE_ARG

struct trace_info
{
	char	*name;
	char	flags;
	char	ret;
	char	argtypes[6];
};

extern int  (*print_arg[])(struct trace_info *, char *, int *, int);

#define	PRINT_ARG(t, buf, args, i)	\
	print_arg[(t)->argtypes[(i)]](t, buf, args, i)

#define	PRINT_RET(t, buf, rval)	\
	print_arg[(t)->ret](t, buf, &rval, 0)


#define	ARG(x)			(a_##x)
#define	RET(x)			ARG(x)
#define	ARGS0()			0
#define	ARGS1(a1)		ARG(a1)
#define	ARGS2(a1,a2)		ARG(a1), ARG(a2)
#define	ARGS3(a1,a2,a3)		ARG(a1), ARG(a2), ARG(a3)
#define	ARGS4(a1,a2,a3,a4)	ARG(a1), ARG(a2), ARG(a3), ARG(a4)
#define	ARGS5(a1,a2,a3,a4,a5)	ARG(a1), ARG(a2), ARG(a3), ARG(a4), ARG(a5)
#define	ARGS6(a1,a2,a3,a4,a5,a6) ARG(a1), ARG(a2), ARG(a3), ARG(a4), ARG(a5), ARG(a6)

/* tracing flags - whats being traced */
#define TRACE_SYS	0x0000000fUL
#define TRACE_SIG	0x000000f0UL
#define TRACE_SOCK	0x00000100UL
#define TRACE_SELECT	0x00000200UL
#define TRACE_READ	0x00000400UL
#define TRACE_WRITE	0x00000800UL
#define TRACE_OPEN	0x00001000UL
#define TRACE_ACCESS	0x00002000UL
#define TRACE_FCNTL	0x00004000UL
#define TRACE_IOCTL	0x00008000UL
#define TRACE_BRK	0x00010000UL
#define TRACE_EXEC	0x00020000UL
#define TRACE_PROCFS	0x00040000UL
#define	TRACE_PATHMAP	0x00080000UL
#define	TRACE_MMAP	0x00100000UL
#define	TRACE_ELF	0x10000000UL
#define TRACE_VERBOSE	0x80000000UL
#define TRACE_ALL	0xffffffffUL

extern unsigned long    do_trace;

extern int		trace_init(const char *element);
extern void		trace_help(void);
extern const char	*lx_sigName(int sig);
extern const char	*os_sigName(int sig);

/* trace display message macros */
#define TRACE_INIT(x)		trace_init(x)
#define SOCK_TRACE_MSG(x)	{ if (do_trace & TRACE_SOCK) tprintf x; }
#define SELECT_TRACE_MSG(x)	{ if (do_trace & TRACE_SELECT) tprintf x; }
#define SIG_TRACE_MSG(x)	{ if (do_trace & TRACE_SIG) tprintf x; }
#define PROCFS_TRACE_MSG(x)	{ if (do_trace & TRACE_PROCFS) tprintf x; }
#define IOCTL_TRACE_MSG(x)	{ if (do_trace & TRACE_IOCTL) tprintf x; }
#define FCNTL_TRACE_MSG(x)	{ if (do_trace & TRACE_FCNTL) tprintf x; }
#define READ_TRACE_MSG(x)	{ if (do_trace & TRACE_READ) tprintf x; }
#define WRITE_TRACE_MSG(x)	{ if (do_trace & TRACE_WRITE) tprintf x; }
#define MMAP_TRACE_MSG(x)	{ if (do_trace & TRACE_MMAP) tprintf x; }
#define PATHMAP_TRACE_MSG(x)	{ if (do_trace & TRACE_PATHMAP) tprintf x; }
#define ELF_TRACE_MSG(x)	{ if (do_trace & TRACE_ELF) tprintf x; }
#define SYS_TRACE_MSG(x)	{ if (do_trace & TRACE_SYS) tprintf x; }
#define TRACE_MSG(t, x)		{ if (do_trace & t)  tprintf x; }

#define SYS_TRACE_CALL(f)	{ if (do_trace & TRACE_SYS)  f; } 
#define TRACE_CALL(t, f)	{ if (do_trace & t)  f; } 

#else	/* TRACE */
#define TRACE_INIT(x)
#define SOCK_TRACE_MSG(x)
#define SELECT_TRACE_MSG(x)
#define SIG_TRACE_MSG(x)
#define PROCFS_TRACE_MSG(x)
#define IOCTL_TRACE_MSG(x)
#define FCNTL_TRACE_MSG(x)
#define READ_TRACE_MSG(x)
#define WRITE_TRACE_MSG(x)
#define MMAP_TRACE_MSG(x)
#define PATHMAP_TRACE_MSG(x)
#define ELF_TRACE_MSG(x)
#define SYS_TRACE_MSG(x)
#define TRACE_MSG(t, x)

#define SYS_TRACE_CALL(f) 
#define TRACE_CALL(t, f)

#endif  /* TRACE_H */
