#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z%%M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */


#include	<errno.h>
#include	<stdarg.h>
#include	<stdio.h>
#include	<string.h>
#include	<limits.h>
#include	<unistd.h>
#include	<sys/time.h>
#include	<sys/reg.h>

#include	"lxrun.h"
#include	"sysent.h"
#include	"linux/ipc.h"
#include	"linux/socket.h"
#include	"linux/fcntl.h"
#include	"linux/signal.h"
#include	"linux/mman.h"
#include	"trace.h"


static int
print_int(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = &buf[strlen(buf)];
    sprintf(p, "%d", args[arg]);
    return 1;
}

static int
print_hex(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = &buf[strlen(buf)];
    sprintf(p, "0x%x", args[arg]);
    return 1;
}

static int
print_oct(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = &buf[strlen(buf)];
    sprintf(p, "0%o", args[arg]);
    return 1;
}


static char *o_accmod[] = { "O_RDONLY", "O_WRONLY", "O_RDWR", "3" };

static struct
{
	int	flag;
	char	*name;
} o_flags [] =
{
    {	LX_O_CREAT,	"O_CREAT"	},
    {	LX_O_EXCL,	"O_EXCL"	},
    {	LX_O_NOCTTY,	"O_NOCTTY"	},
    {	LX_O_TRUNC,	"O_TRUNC"	},
    {	LX_O_APPEND,	"O_APPEND"	},
    {	LX_O_NONBLOCK,	"O_NONBLOCK"	},
    {	LX_O_SYNC,	"O_SYNC"	},
    {	0,		0		}
};


static int
print_oflags(struct trace_info *t, char *buf, int *args, int arg)
{
    char	*p = &buf[strlen(buf)];
    int		i;
    int		flags = args[arg];

    p += sprintf(p, o_accmod[flags & LX_O_ACCMOD]);
    flags &= ~LX_O_ACCMOD;

    for (i = 0; o_flags[i].flag != 0 && flags != 0; i++) {
	if (o_flags[i].flag & flags) {
	    p += sprintf(p, "|%s", o_flags[i].name);
	    flags &= ~o_flags[i].flag;
	}
    }

    if (flags)
	p += sprintf(p, "|0x%x", flags);

    if (args[arg] & LX_O_CREAT)
	p += sprintf(p, ", 0%o", args[arg+1]);
	
    return 2;
}

static int
print_ptr(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = &buf[strlen(buf)];
    sprintf(p, "0x%x", args[arg]);
    return 1;
}

static int
print_path(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = buf + strlen(buf);
    char *str = (char *)args[arg];

    if (verify_readstr(str) == 0)
	sprintf(p, "0x%p", (void *)str);
    else
    {
	p += sprintf(p, "\"%.60s\"", str);
	/* trace path mapping in lx_pathname if enabled */
    }
    return 1;
}

static int
print_nmpath(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = buf + strlen(buf);
    char *str = (char *)args[arg];

    if (verify_readstr(str) == 0)
	sprintf(p, "0x%p", (void *)str);
    else
	sprintf(p, "\"%.60s\"", str);
    return 1;
}

static int
print_str(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = &buf[strlen(buf)];
    char *str = (char *)args[arg];

    if (verify_readstr(str) == 0)
	sprintf(p, "0x%p", (void *)str);
    else
	sprintf(p, "%.20s", str);
    return 1;
}

static int
print_sig(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = &buf[strlen(buf)];
    sprintf(p, "%d", args[arg]);
    return 1;
}


static int
print_socketcall(struct trace_info *t, char *buf, int *args, int arg)
{
	int	n;
	extern struct trace_info socket_info[];

	if ((n = args[0]) > 0 && n <= 15)
	{
		int	i;
		struct trace_info	*t;
		t = &socket_info[n];
		sprintf(buf, "%s(", t->name);

		for (i = 0; i < 6 && t->argtypes[i] != 0; i++)
		{
			if (i > 0)
				strcat(buf, ", ");
			PRINT_ARG(t, buf, (int *)args[1], i);
		}
	}
	else
	{
		char *p = &buf[strlen(buf)];
		sprintf(p, "%d", args[arg]);
	}
	return 1;
}

static int
print_ipc(struct trace_info *t, char *buf, int *args, int arg)
{
    char *p = &buf[strlen(buf)];
    sprintf(p, "%p", (void*)args[arg]);
    return 1;
}

static struct trace_info select_info =
{ "select", 0, RET(int), { ARGS5(int, fd_set, fd_set, fd_set, tv) } };

static int
print_old_select(struct trace_info *t, char *buf, int *args, int arg)
{
	int	*av	= (int *)args[0];
	int	i;

	if (! verify_read(av, 5 * sizeof(av[0]))) {
		strcat(buf, "<**fault**>");
		return 1;
	}

	t = &select_info;
	sprintf(buf, "%s(", t->name);
	for (i = 0; i < 6 && t->argtypes[i] != 0; i++)
	{
		if (i > 0)
			strcat(buf, ", ");
		PRINT_ARG(t, buf, av, i);
	}

	return 1;
}

static int
print_none(struct trace_info *t, char *buf, int *args, int arg)
{
    return 1;
}


static int
print_tv(struct trace_info *t, char *buf, int *args, int arg)
{
	struct timeval *tv;

	if ((tv = (struct timeval *)args[arg]) == NULL)
		strcat(buf, "NULL");
	else if (! verify_read(tv, sizeof(*tv)))
		strcat(buf, "<**fault**>");
	else {
		char *p = &buf[strlen(buf)];
		sprintf(p, "{%ld,%ld}", tv->tv_sec, tv->tv_usec);
	}

    return 1;
}


static int
print_itv(struct trace_info *t, char *buf, int *args, int arg)
{
	struct itimerval	*itv;
	struct timeval		*tvargs[2];

	itv = (struct itimerval *)args[arg];

	tvargs[0] = &itv->it_interval;
	tvargs[1] = &itv->it_value;
	print_tv(t, buf, (int *)tvargs, 0);
	print_tv(t, buf, (int *)tvargs, 1);	
	return 1;
}

static void
print_bitmask(char *buf, unsigned char *bitp, int nbits, int origin)
{
	int	b	= 0;
	int	n	= 0;
	int	count	= (nbits + (CHAR_BIT-1)) / CHAR_BIT;
	int	first	= -1;
	int	flag	= 0;
	char	*p	= &buf[strlen(buf)];

	if (bitp == NULL) {
		strcpy(p, "NULL");
		return;
	}

	if (! verify_read(bitp, count)) {
		strcpy(p, "<**fault**>");
		return;
	}

	*p++	= '<';

	while (nbits > 0) {
		unsigned char bits = *bitp++;
		unsigned char mask;

		n      = (nbits > CHAR_BIT) ? CHAR_BIT : nbits;
		nbits -= n;

		for (mask = 1; --n >= 0; mask <<= 1, b++) {
			if (bits & mask) {
				if (first < 0)
					first = b;
			}
			else {
				if (first >= 0) {
					if (flag++)
						*p++ = ',';

					p += sprintf(p, "%d", first + origin);
					if (first != (b-1))
						p += sprintf(p, "-%d",
							(b-1+origin));
					first = -1;
				}
			}
		}
	}

	if (first >= 0) {
		if (flag++)
			*p++ = ',';

		p += sprintf(p, "%d", first + origin);
		if (first != (b-1))
			p += sprintf(p, "-%d", (b-1+origin));
		first = -1;
	}

	strcpy(p, flag ? ">" : "empty>");

}

static int
print_fd_set(struct trace_info *t, char *buf, int *args, int arg)
{
	print_bitmask(buf, (unsigned char *)args[arg], args[0], 0);
	return 1;
}

static int
print_sigset(struct trace_info *t, char *buf, int *args, int arg)
{
	print_bitmask(buf, (unsigned char *)args[arg], LX_NSIG, 1);
	return 1;
}

static int
print_sigact(struct trace_info *t, char *buf, int *args, int arg)
{
	struct lx_sigaction	*sa	= (struct lx_sigaction *)args[arg];
	char			*p	= &buf[strlen(buf)];

	if (sa == NULL) {
		strcpy(p, "NULL");
		return 1;
	}

	if (! verify_read(sa, sizeof(*sa))) {
		strcpy(p, "<**fault**>");
		return 1;
	}

	if (sa->handler == LX_SIG_DFL)
		p += sprintf(p, "{ SIG_DFL, ");
	else if (sa->handler == LX_SIG_IGN)
		p += sprintf(p, "{ SIG_IGN, ");
	else
		p += sprintf(p, "{ 0x%lx, ", (unsigned long)sa->handler);

	print_bitmask(buf, (unsigned char *)&sa->mask, LX_NSIG, 1);

	p = &buf[strlen(buf)];
	sprintf(p, ", 0x%lx }", sa->flags);

	return 1;
}

static int
print_timespec(struct trace_info *t, char *buf, int *args, int arg)
{
#ifdef SOLARIS
	struct timespec *tv;

	if ((tv = (struct timespec *)args[arg]) == NULL)
		strcat(buf, "NULL");
	else if (! verify_read(tv, sizeof(*tv)))
		strcat(buf, "<**fault**>");
	else {
		char *p = &buf[strlen(buf)];
		sprintf(p, "{%ld, %ld}", tv->tv_sec, tv->tv_nsec);
	}

	return 1;
#endif
}

static int
print_mmap(struct trace_info *t, char *buf, int *args, int arg)
{
	unsigned long *av = (unsigned long *)args[arg];

	if (!verify_read(av, 6 * sizeof (av[0]))) {
		strcat(buf, "<**fault**>");
	} else {
		int prot = (int)av[2];
		int flags = (int)av[3];
		char *p = buf + strlen(buf);

		p += sprintf(p, "0x%x, %u,", (void *)av[0], (size_t)av[1]);

		if (prot & (LX_PROT_READ | LX_PROT_WRITE | LX_PROT_EXEC) == 0) {
			p += sprintf(p, " PROT_NONE");
		} else {
			char *q = p;
			if (prot & LX_PROT_READ)
				p += sprintf(p, "|PROT_READ");
			if (prot & LX_PROT_WRITE)
				p += sprintf(p, "|PROT_WRITE");
			if (prot & LX_PROT_EXEC)
				p += sprintf(p, "|PROT_EXEC");
			*q++ = ' ';
		}
		p += sprintf(p, ", ");

		switch (flags & LX_MAP_TYPE) {
			case LX_MAP_SHARED:
				p += sprintf(p, "MAP_SHARED");
				break;
			case LX_MAP_PRIVATE:
				p += sprintf(p, "MAP_PRIVATE");
				break;
			default:
				p += sprintf(p, "0x%x", flags & LX_MAP_TYPE);
				break;
		}
		if (flags & LX_MAP_FIXED)
			p += sprintf(p, "|MAP_FIXED");
		if (flags & LX_MAP_ANONYMOUS)
			p += sprintf(p, "|MAP_ANONYMOUS");
		if (flags & LX_MAP_GROWSDOWN)
			p += sprintf(p, "|MAP_GROWSDOWN");
		if (flags & LX_MAP_DENYWRITE)
			p += sprintf(p, "|MAP_DENYWRITE");
		if (flags & LX_MAP_EXECUTABLE)
			p += sprintf(p, "|MAP_EXECUTABLE");

		p += sprintf(p, ", %d, %d", (int)av[4], (off_t)av[5]);
	}
	
	return 1;
}

/*
 * most of the argument printing functions are not implemented
 * yet so they get mapped onto something generic
 */
#define print_dev	print_hex
#define print_fd	print_int
#define print_gid	print_int
#define print_uid	print_int
#define print_size	print_int
#define print_pid	print_int
#define print_buf	print_ptr
#define print_buf_r	print_ptr
#define print_buf_w	print_ptr
#define print_iov_r	print_ptr
#define print_iov_w	print_ptr
#define print_res	print_int
#define print_rlim	print_ptr
#define print_rusage	print_ptr
#define print_tz	print_ptr
#define print_statfs	print_ptr
#define print_stat	print_ptr
#define print_prot	print_hex
#define print_sched_param	print_ptr
#define print_ts	print_ptr
#define print_mode	print_oct
#define print_argv	print_ptr
#define print_old_stat	print_ptr
#define print_off	print_int
#define print_amode	print_oct
#define print_si	print_int
#define print_pipefds	print_ptr
#define print_tms	print_ptr

#undef TRACE_ARG
#define	TRACE_ARG(atype)	print_##atype,

int   (*print_arg[])(struct trace_info *, char *, int *, int) =
{
#include "traceargs.h"
};

