#ifdef OSR5
#pragma comment(exestr, "%Z% %M% %I% %E% %Q%")
#endif

#ifdef SOLARIS
#pragma ident	"%Z%%M% %I% %D% SMI"
#endif
/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#include	<errno.h>
#include	<stdlib.h>
#include	"lxrun.h"

int
verify_read(const void *addr, size_t len)
{
	const unsigned char	*start		= addr;
	const unsigned char	*end		= start + len - 1;
	const unsigned char	*p;

	for (p = start; p <= end; p += PAGE_SIZE)
		if (getbyte(p) < 0)
		{
			errno = EFAULT;
			return 0;
		}

	return 1;
}

int
verify_write(void *addr, size_t len)
{
	unsigned char	*start		= addr;
	unsigned char	*end		= start + len - 1;
	unsigned char	*p;
	int		c;

	for (p = start; p <= end; p += PAGE_SIZE)
		if ((c = getbyte(p)) < 0 || putbyte(p, c) < 0)
		{
			errno = EFAULT;
			return 0;
		}

	return 1;
}

int
verify_readstr(const char *addr)
{
	const char *p = addr;
	const char *pagep;

	do {
		if (getbyte(p) < 0) {
			errno = EFAULT;
			return 0;
		}
		/*
		 * pagep = "base of next page" - different from PAGE_ALIGN().
		 */
		pagep = (const char *)((((uintptr_t)p) & ~(PAGE_SIZE - 1))
		    + PAGE_SIZE);
		while (p < pagep && *p != 0)
			p++;
	} while (p == pagep);

	return 1;
}
